@@ -40,7 +40,7 @@ exchange rates.
 
 -}
 
-{-# LANGUAGE CPP, StandaloneDeriving, RecordWildCards, OverloadedStrings #-}
+{-# LANGUAGE StandaloneDeriving, RecordWildCards, OverloadedStrings #-}
 
 module Hledger.Data.Amount (
   -- * Amount
@@ -58,7 +58,11 @@ module Hledger.Data.Amount (
   -- ** arithmetic
   costOfAmount,
   divideAmount,
+  multiplyAmount,
+  divideAmountAndPrice,
+  multiplyAmountAndPrice,
   amountValue,
+  amountTotalPriceToUnitPrice,
   -- ** rendering
   amountstyle,
   styleAmount,
@@ -71,6 +75,10 @@ module Hledger.Data.Amount (
   maxprecisionwithpoint,
   setAmountPrecision,
   withPrecision,
+  setAmountInternalPrecision,
+  withInternalPrecision,
+  setAmountDecimalPoint,
+  withDecimalPoint,
   canonicaliseAmount,
   -- * MixedAmount
   nullmixedamt,
@@ -84,6 +92,9 @@ module Hledger.Data.Amount (
   -- ** arithmetic
   costOfMixedAmount,
   divideMixedAmount,
+  multiplyMixedAmount,
+  divideMixedAmountAndPrice,
+  multiplyMixedAmountAndPrice,
   averageMixedAmounts,
   isNegativeAmount,
   isNegativeMixedAmount,
@@ -93,6 +104,7 @@ module Hledger.Data.Amount (
   isReallyZeroMixedAmount,
   isReallyZeroMixedAmountCost,
   mixedAmountValue,
+  mixedAmountTotalPriceToUnitPrice,
   -- ** rendering
   styleMixedAmount,
   showMixedAmount,
@@ -108,7 +120,7 @@ module Hledger.Data.Amount (
   canonicaliseMixedAmount,
   -- * misc.
   ltraceamount,
-  tests_Hledger_Data_Amount
+  tests_Amount
 ) where
 
 import Data.Char (isDigit)
@@ -122,13 +134,12 @@ import Data.Ord (comparing)
 -- import Data.Text (Text)
 import qualified Data.Text as T
 import Safe (maximumDef)
-import Test.HUnit
 import Text.Printf
 import qualified Data.Map as M
 
 import Hledger.Data.Types
 import Hledger.Data.Commodity
-import Hledger.Utils
+import Hledger.Utils 
 
 
 deriving instance Show MarketPrice
@@ -144,15 +155,6 @@ amountstyle = AmountStyle L False 0 (Just '.') Nothing
 -------------------------------------------------------------------------------
 -- Amount
 
-instance Show Amount where
-  show _a@Amount{..}
-    --  debugLevel < 2 = showAmountWithoutPrice a
-    --  debugLevel < 3 = showAmount a
-    | debugLevel < 6 =
-       printf "Amount {acommodity=%s, aquantity=%s, ..}" (show acommodity) (show aquantity)
-    | otherwise      = --showAmountDebug a
-       printf "Amount {acommodity=%s, aquantity=%s, aprice=%s, astyle=%s}" (show acommodity) (show aquantity) (showPriceDebug aprice) (show astyle)
-
 instance Num Amount where
     abs a@Amount{aquantity=q}    = a{aquantity=abs q}
     signum a@Amount{aquantity=q} = a{aquantity=signum q}
@@ -213,9 +215,40 @@ costOfAmount a@Amount{aquantity=q, aprice=price} =
       UnitPrice  p@Amount{aquantity=pq} -> p{aquantity=pq * q}
       TotalPrice p@Amount{aquantity=pq} -> p{aquantity=pq * signum q}
 
+-- | Replace an amount's TotalPrice, if it has one, with an equivalent UnitPrice.
+-- Has no effect on amounts without one.
+-- Also increases the unit price's display precision to show one extra decimal place,
+-- to help keep transaction amounts balancing. 
+-- Does Decimal division, might be some rounding/irrational number issues.
+amountTotalPriceToUnitPrice :: Amount -> Amount
+amountTotalPriceToUnitPrice 
+  a@Amount{aquantity=q, aprice=TotalPrice pa@Amount{aquantity=pq, astyle=ps@AmountStyle{asprecision=pp}}}
+  = a{aprice = UnitPrice pa{aquantity=abs (pq/q), astyle=ps{asprecision=pp+1}}}
+amountTotalPriceToUnitPrice a = a
+
 -- | Divide an amount's quantity by a constant.
-divideAmount :: Amount -> Quantity -> Amount
-divideAmount a@Amount{aquantity=q} d = a{aquantity=q/d}
+divideAmount :: Quantity -> Amount -> Amount
+divideAmount n a@Amount{aquantity=q} = a{aquantity=q/n}
+
+-- | Multiply an amount's quantity by a constant.
+multiplyAmount :: Quantity -> Amount -> Amount
+multiplyAmount n a@Amount{aquantity=q} = a{aquantity=q*n}
+
+-- | Divide an amount's quantity (and its total price, if it has one) by a constant.
+-- The total price will be kept positive regardless of the multiplier's sign.
+divideAmountAndPrice :: Quantity -> Amount -> Amount
+divideAmountAndPrice n a@Amount{aquantity=q,aprice=p} = a{aquantity=q/n, aprice=f p}
+  where
+    f (TotalPrice a) = TotalPrice $ abs $ n `divideAmount` a
+    f p = p
+
+-- | Multiply an amount's quantity (and its total price, if it has one) by a constant.
+-- The total price will be kept positive regardless of the multiplier's sign.
+multiplyAmountAndPrice :: Quantity -> Amount -> Amount
+multiplyAmountAndPrice n a@Amount{aquantity=q,aprice=p} = a{aquantity=q*n, aprice=f p}
+  where
+    f (TotalPrice a) = TotalPrice $ abs $ n `multiplyAmount` a
+    f p = p
 
 -- | Is this amount negative ? The price is ignored.
 isNegativeAmount :: Amount -> Bool
@@ -255,6 +288,31 @@ showAmountDebug Amount{..} = printf "Amount {acommodity=%s, aquantity=%s, aprice
 showAmountWithoutPrice :: Amount -> String
 showAmountWithoutPrice a = showAmount a{aprice=NoPrice}
 
+-- | Set an amount's internal precision, ie rounds the Decimal representing 
+-- the amount's quantity to some number of decimal places.
+-- Rounding is done with Data.Decimal's default roundTo function:
+-- "If the value ends in 5 then it is rounded to the nearest even value (Banker's Rounding)".
+-- Does not change the amount's display precision.
+-- Intended only for internal use, eg when comparing amounts in tests. 
+setAmountInternalPrecision :: Int -> Amount -> Amount
+setAmountInternalPrecision p a@Amount{ aquantity=q, astyle=s } = a{ 
+   astyle=s{asprecision=p} 
+  ,aquantity=roundTo (fromIntegral p) q
+  }
+
+-- | Set an amount's internal precision, flipped.
+-- Intended only for internal use, eg when comparing amounts in tests. 
+withInternalPrecision :: Amount -> Int -> Amount
+withInternalPrecision = flip setAmountInternalPrecision
+
+-- | Set (or clear) an amount's display decimal point.
+setAmountDecimalPoint :: Maybe Char -> Amount -> Amount
+setAmountDecimalPoint mc a@Amount{ astyle=s } = a{ astyle=s{asdecimalpoint=mc} }
+
+-- | Set (or clear) an amount's display decimal point, flipped.
+withDecimalPoint :: Amount -> Maybe Char -> Amount
+withDecimalPoint = flip setAmountDecimalPoint
+
 -- | Colour version.
 cshowAmountWithoutPrice :: Amount -> String
 cshowAmountWithoutPrice a = cshowAmount a{aprice=NoPrice}
@@ -375,9 +433,7 @@ canonicaliseAmount styles a@Amount{acommodity=c, astyle=s} = a{astyle=s'}
 amountValue :: Journal -> Day -> Amount -> Amount
 amountValue j d a =
   case commodityValue j d (acommodity a) of
-    Just v  -> v{aquantity=aquantity v * aquantity a
-                ,aprice=aprice a
-                }
+    Just v  -> v{aquantity=aquantity v * aquantity a}
     Nothing -> a
 
 -- This is here not in Commodity.hs to use the Amount Show instance above for debugging. 
@@ -403,12 +459,6 @@ commodityValue j valuationdate c
 -------------------------------------------------------------------------------
 -- MixedAmount
 
-instance Show MixedAmount where
-  show
-    | debugLevel < 3 = intercalate "\\n" . lines . showMixedAmountWithoutPrice
-    --  debugLevel < 6 = intercalate "\\n" . lines . showMixedAmount
-    | otherwise      = showMixedAmountDebug
-
 instance Num MixedAmount where
     fromInteger i = Mixed [fromInteger i]
     negate (Mixed as) = Mixed $ map negate as
@@ -467,36 +517,12 @@ normaliseHelper squashprices (Mixed as)
     combinableprices Amount{aprice=UnitPrice p1} Amount{aprice=UnitPrice p2} = p1 == p2
     combinableprices _ _ = False
 
-tests_normaliseMixedAmount = [
-  "normaliseMixedAmount" ~: do
-   -- assertEqual "missing amount is discarded" (Mixed [nullamt]) (normaliseMixedAmount $ Mixed [usd 0, missingamt])
-   assertEqual "any missing amount means a missing mixed amount" missingmixedamt (normaliseMixedAmount $ Mixed [usd 0, missingamt])
-   assertEqual "unpriced same-commodity amounts are combined" (Mixed [usd 2]) (normaliseMixedAmount $ Mixed [usd 0, usd 2])
-   -- amounts with same unit price are combined
-   normaliseMixedAmount (Mixed [usd 1 `at` eur 1, usd 1 `at` eur 1]) `is` Mixed [usd 2 `at` eur 1]
-   -- amounts with different unit prices are not combined
-   normaliseMixedAmount (Mixed [usd 1 `at` eur 1, usd 1 `at` eur 2]) `is` Mixed [usd 1 `at` eur 1, usd 1 `at` eur 2]
-   -- amounts with total prices are not combined
-   normaliseMixedAmount (Mixed  [usd 1 @@ eur 1, usd 1 @@ eur 1]) `is` Mixed [usd 1 @@ eur 1, usd 1 @@ eur 1]
- ]
-
 -- | Like normaliseMixedAmount, but combine each commodity's amounts
 -- into just one by throwing away all prices except the first. This is
 -- only used as a rendering helper, and could show a misleading price.
 normaliseMixedAmountSquashPricesForDisplay :: MixedAmount -> MixedAmount
 normaliseMixedAmountSquashPricesForDisplay = normaliseHelper True
 
-tests_normaliseMixedAmountSquashPricesForDisplay = [
-  "normaliseMixedAmountSquashPricesForDisplay" ~: do
-    normaliseMixedAmountSquashPricesForDisplay (Mixed []) `is` Mixed [nullamt]
-    assertBool "" $ isZeroMixedAmount $ normaliseMixedAmountSquashPricesForDisplay
-      (Mixed [usd 10
-             ,usd 10 @@ eur 7
-             ,usd (-10)
-             ,usd (-10) @@ eur 7
-             ])
- ]
-
 -- | Sum same-commodity amounts in a lossy way, applying the first
 -- price to the result and discarding any other prices. Only used as a
 -- rendering helper.
@@ -530,19 +556,37 @@ filterMixedAmountByCommodity c (Mixed as) = Mixed as'
             []   -> [nullamt{acommodity=c}]
             as'' -> [sum as'']
 
+-- | Apply a transform to a mixed amount's component 'Amount's.
+mapMixedAmount :: (Amount -> Amount) -> MixedAmount -> MixedAmount
+mapMixedAmount f (Mixed as) = Mixed $ map f as
+
 -- | Convert a mixed amount's component amounts to the commodity of their
 -- assigned price, if any.
 costOfMixedAmount :: MixedAmount -> MixedAmount
 costOfMixedAmount (Mixed as) = Mixed $ map costOfAmount as
 
 -- | Divide a mixed amount's quantities by a constant.
-divideMixedAmount :: MixedAmount -> Quantity -> MixedAmount
-divideMixedAmount (Mixed as) d = Mixed $ map (`divideAmount` d) as
+divideMixedAmount :: Quantity -> MixedAmount -> MixedAmount
+divideMixedAmount n = mapMixedAmount (divideAmount n)
+
+-- | Multiply a mixed amount's quantities by a constant.
+multiplyMixedAmount :: Quantity -> MixedAmount -> MixedAmount
+multiplyMixedAmount n = mapMixedAmount (multiplyAmount n)
+
+-- | Divide a mixed amount's quantities (and total prices, if any) by a constant.
+-- The total prices will be kept positive regardless of the multiplier's sign.
+divideMixedAmountAndPrice :: Quantity -> MixedAmount -> MixedAmount
+divideMixedAmountAndPrice n = mapMixedAmount (divideAmountAndPrice n)
+
+-- | Multiply a mixed amount's quantities (and total prices, if any) by a constant.
+-- The total prices will be kept positive regardless of the multiplier's sign.
+multiplyMixedAmountAndPrice :: Quantity -> MixedAmount -> MixedAmount
+multiplyMixedAmountAndPrice n = mapMixedAmount (multiplyAmountAndPrice n)
 
 -- | Calculate the average of some mixed amounts.
 averageMixedAmounts :: [MixedAmount] -> MixedAmount
 averageMixedAmounts [] = 0
-averageMixedAmounts as = sum as `divideMixedAmount` fromIntegral (length as)
+averageMixedAmounts as = fromIntegral (length as) `divideMixedAmount` sum as 
 
 -- | Is this mixed amount negative, if it can be normalised to a single commodity ?
 isNegativeMixedAmount :: MixedAmount -> Maybe Bool
@@ -668,82 +712,113 @@ canonicaliseMixedAmount styles (Mixed as) = Mixed $ map (canonicaliseAmount styl
 mixedAmountValue :: Journal -> Day -> MixedAmount -> MixedAmount
 mixedAmountValue j d (Mixed as) = Mixed $ map (amountValue j d) as
 
+-- | Replace each component amount's TotalPrice, if it has one, with an equivalent UnitPrice.
+-- Has no effect on amounts without one. 
+-- Does Decimal division, might be some rounding/irrational number issues.
+mixedAmountTotalPriceToUnitPrice :: MixedAmount -> MixedAmount
+mixedAmountTotalPriceToUnitPrice (Mixed as) = Mixed $ map amountTotalPriceToUnitPrice as
+
 
 -------------------------------------------------------------------------------
--- misc
+-- tests
 
-tests_Hledger_Data_Amount = TestList $
-     tests_normaliseMixedAmount
-  ++ tests_normaliseMixedAmountSquashPricesForDisplay
-  ++ [
+tests_Amount = tests "Amount" [
+   tests "Amount" [
 
-  -- Amount
-
-   "costOfAmount" ~: do
-    costOfAmount (eur 1) `is` eur 1
-    costOfAmount (eur 2){aprice=UnitPrice $ usd 2} `is` usd 4
-    costOfAmount (eur 1){aprice=TotalPrice $ usd 2} `is` usd 2
-    costOfAmount (eur (-1)){aprice=TotalPrice $ usd 2} `is` usd (-2)
-
-  ,"isZeroAmount" ~: do
-    assertBool "" $ isZeroAmount amount
-    assertBool "" $ isZeroAmount $ usd 0
-
-  ,"negating amounts" ~: do
-    let a = usd 1
-    negate a `is` a{aquantity= -1}
-    let b = (usd 1){aprice=UnitPrice $ eur 2}
-    negate b `is` b{aquantity= -1}
-
-  ,"adding amounts without prices" ~: do
-    let a1 = usd 1.23
-    let a2 = usd (-1.23)
-    let a3 = usd (-1.23)
-    (a1 + a2) `is` usd 0
-    (a1 + a3) `is` usd 0
-    (a2 + a3) `is` usd (-2.46)
-    (a3 + a3) `is` usd (-2.46)
-    sum [a1,a2,a3,-a3] `is` usd 0
-    -- highest precision is preserved
-    let ap1 = usd 1 `withPrecision` 1
-        ap3 = usd 1 `withPrecision` 3
-    asprecision (astyle $ sum [ap1,ap3]) `is` 3
-    asprecision (astyle $ sum [ap3,ap1]) `is` 3
-    -- adding different commodities assumes conversion rate 1
-    assertBool "" $ isZeroAmount (a1 - eur 1.23)
-
-  ,"showAmount" ~: do
-    showAmount (usd 0 + gbp 0) `is` "0"
-
-  -- MixedAmount
-
-  ,"adding mixed amounts to zero, the commodity and amount style are preserved" ~: do
-    sum (map (Mixed . (:[]))
-             [usd 1.25
-             ,usd (-1) `withPrecision` 3
-             ,usd (-0.25)
-             ])
-      `is` Mixed [usd 0 `withPrecision` 3]
-
-  ,"adding mixed amounts with total prices" ~: do
-    sum (map (Mixed . (:[]))
-     [usd 1 @@ eur 1
-     ,usd (-2) @@ eur 1
-     ])
-      `is` Mixed [usd 1 @@ eur 1
-                 ,usd (-2) @@ eur 1
-                 ]
-
-  ,"showMixedAmount" ~: do
-    showMixedAmount (Mixed [usd 1]) `is` "$1.00"
-    showMixedAmount (Mixed [usd 1 `at` eur 2]) `is` "$1.00 @ €2.00"
-    showMixedAmount (Mixed [usd 0]) `is` "0"
-    showMixedAmount (Mixed []) `is` "0"
-    showMixedAmount missingmixedamt `is` ""
-
-  ,"showMixedAmountWithoutPrice" ~: do
-    let a = usd 1 `at` eur 2
-    showMixedAmountWithoutPrice (Mixed [a]) `is` "$1.00"
-    showMixedAmountWithoutPrice (Mixed [a, -a]) `is` "0"
+     tests "costOfAmount" [
+       costOfAmount (eur 1) `is` eur 1
+      ,costOfAmount (eur 2){aprice=UnitPrice $ usd 2} `is` usd 4
+      ,costOfAmount (eur 1){aprice=TotalPrice $ usd 2} `is` usd 2
+      ,costOfAmount (eur (-1)){aprice=TotalPrice $ usd 2} `is` usd (-2)
+    ]
+  
+    ,tests "isZeroAmount" [
+       expect $ isZeroAmount amount
+      ,expect $ isZeroAmount $ usd 0
+    ]
+  
+    ,tests "negating amounts" [
+       negate (usd 1) `is` (usd 1){aquantity= -1}
+      ,let b = (usd 1){aprice=UnitPrice $ eur 2} in negate b `is` b{aquantity= -1}
+    ]
+  
+    ,tests "adding amounts without prices" [
+       (usd 1.23 + usd (-1.23)) `is` usd 0
+      ,(usd 1.23 + usd (-1.23)) `is` usd 0
+      ,(usd (-1.23) + usd (-1.23)) `is` usd (-2.46)
+      ,sum [usd 1.23,usd (-1.23),usd (-1.23),-(usd (-1.23))] `is` usd 0
+      -- highest precision is preserved
+      ,asprecision (astyle $ sum [usd 1 `withPrecision` 1, usd 1 `withPrecision` 3]) `is` 3
+      ,asprecision (astyle $ sum [usd 1 `withPrecision` 3, usd 1 `withPrecision` 1]) `is` 3
+      -- adding different commodities assumes conversion rate 1
+      ,expect $ isZeroAmount (usd 1.23 - eur 1.23)
+    ]
+  
+    ,tests "showAmount" [
+      showAmount (usd 0 + gbp 0) `is` "0"
+    ]
 
   ]
+
+  ,tests "MixedAmount" [
+
+     tests "adding mixed amounts to zero, the commodity and amount style are preserved" [
+      sum (map (Mixed . (:[]))
+               [usd 1.25
+               ,usd (-1) `withPrecision` 3
+               ,usd (-0.25)
+               ])
+        `is` Mixed [usd 0 `withPrecision` 3]
+    ]
+  
+    ,tests "adding mixed amounts with total prices" [
+      sum (map (Mixed . (:[]))
+       [usd 1 @@ eur 1
+       ,usd (-2) @@ eur 1
+       ])
+        `is` Mixed [usd 1 @@ eur 1
+                   ,usd (-2) @@ eur 1
+                   ]
+    ]
+  
+    ,tests "showMixedAmount" [
+       showMixedAmount (Mixed [usd 1]) `is` "$1.00"
+      ,showMixedAmount (Mixed [usd 1 `at` eur 2]) `is` "$1.00 @ €2.00"
+      ,showMixedAmount (Mixed [usd 0]) `is` "0"
+      ,showMixedAmount (Mixed []) `is` "0"
+      ,showMixedAmount missingmixedamt `is` ""
+    ]
+  
+    ,tests "showMixedAmountWithoutPrice" $
+      let a = usd 1 `at` eur 2 in 
+    [
+        showMixedAmountWithoutPrice (Mixed [a]) `is` "$1.00"
+       ,showMixedAmountWithoutPrice (Mixed [a, -a]) `is` "0"
+    ]
+  
+    ,tests "normaliseMixedAmount" [
+       test "a missing amount overrides any other amounts" $ 
+        normaliseMixedAmount (Mixed [usd 1, missingamt]) `is` missingmixedamt
+      ,test "unpriced same-commodity amounts are combined" $ 
+        normaliseMixedAmount (Mixed [usd 0, usd 2]) `is` Mixed [usd 2]
+      ,test "amounts with same unit price are combined" $ 
+        normaliseMixedAmount (Mixed [usd 1 `at` eur 1, usd 1 `at` eur 1]) `is` Mixed [usd 2 `at` eur 1]
+      ,test "amounts with different unit prices are not combined" $ 
+        normaliseMixedAmount (Mixed [usd 1 `at` eur 1, usd 1 `at` eur 2]) `is` Mixed [usd 1 `at` eur 1, usd 1 `at` eur 2]
+      ,test "amounts with total prices are not combined" $
+        normaliseMixedAmount (Mixed  [usd 1 @@ eur 1, usd 1 @@ eur 1]) `is` Mixed [usd 1 @@ eur 1, usd 1 @@ eur 1]
+    ]
+  
+    ,tests "normaliseMixedAmountSquashPricesForDisplay" [
+       normaliseMixedAmountSquashPricesForDisplay (Mixed []) `is` Mixed [nullamt]
+      ,expect $ isZeroMixedAmount $ normaliseMixedAmountSquashPricesForDisplay
+        (Mixed [usd 10
+               ,usd 10 @@ eur 7
+               ,usd (-10)
+               ,usd (-10) @@ eur 7
+               ])
+    ]
+
+  ]
+
+ ]
