@@ -48,6 +48,7 @@ module Hledger.Data.Dates (
   parsePeriodExpr,
   parsePeriodExpr',
   nulldatespan,
+  emptydatespan,
   failIfInvalidYear,
   failIfInvalidMonth,
   failIfInvalidDay,
@@ -77,6 +78,7 @@ where
 
 import Prelude ()
 import "base-compat-batteries" Prelude.Compat
+import Control.Applicative.Permutations
 import Control.Monad
 import "base-compat-batteries" Data.List.Compat
 import Data.Default
@@ -96,7 +98,7 @@ import Data.Time.LocalTime
 import Safe (headMay, lastMay, readMay)
 import Text.Megaparsec
 import Text.Megaparsec.Char
-import Text.Megaparsec.Perm
+import Text.Megaparsec.Custom
 import Text.Printf
 
 import Hledger.Data.Types
@@ -205,7 +207,7 @@ splitSpan (Years n)      s = splitspan startofyear    (applyN n nextyear)    s
 splitSpan (DayOfMonth n) s = splitspan (nthdayofmonthcontaining n) (nthdayofmonth n . nextmonth) s
 splitSpan (WeekdayOfMonth n wd) s = splitspan (nthweekdayofmonthcontaining n wd) (advancetonthweekday n wd . nextmonth) s
 splitSpan (DayOfWeek n)  s = splitspan (nthdayofweekcontaining n)  (applyN (n-1) nextday . nextweek)  s
-splitSpan (DayOfYear m n) s= splitspan (nthdayofyearcontaining m n) (applyN (n-1) nextday . applyN (m-1) nextmonth . nextyear) s
+splitSpan (DayOfYear m n) s = splitspan (nthdayofyearcontaining m n) (applyN (n-1) nextday . applyN (m-1) nextmonth . nextyear) s
 -- splitSpan (WeekOfYear n)    s = splitspan startofweek    (applyN n nextweek)    s
 -- splitSpan (MonthOfYear n)   s = splitspan startofmonth   (applyN n nextmonth)   s
 -- splitSpan (QuarterOfYear n) s = splitspan startofquarter (applyN n nextquarter) s
@@ -314,13 +316,14 @@ earliest (Just d1) (Just d2) = Just $ min d1 d2
 
 -- | Parse a period expression to an Interval and overall DateSpan using
 -- the provided reference date, or return a parse error.
-parsePeriodExpr :: Day -> Text -> Either (ParseError Char CustomErr) (Interval, DateSpan)
+parsePeriodExpr
+  :: Day -> Text -> Either (ParseErrorBundle Text CustomErr) (Interval, DateSpan)
 parsePeriodExpr refdate s = parsewith (periodexprp refdate <* eof) (T.toLower s)
 
 -- | Like parsePeriodExpr, but call error' on failure.
 parsePeriodExpr' :: Day -> Text -> (Interval, DateSpan)
 parsePeriodExpr' refdate s =
-  either (error' . ("failed to parse:" ++) . parseErrorPretty) id $
+  either (error' . ("failed to parse:" ++) . customErrorBundlePretty) id $
   parsePeriodExpr refdate s
 
 maybePeriod :: Day -> Text -> Maybe (Interval,DateSpan)
@@ -380,13 +383,14 @@ fixSmartDateStr :: Day -> Text -> String
 fixSmartDateStr d s = either
                        (\e->error' $ printf "could not parse date %s %s" (show s) (show e))
                        id
-                       $ (fixSmartDateStrEither d s :: Either (ParseError Char CustomErr) String)
+                       $ (fixSmartDateStrEither d s :: Either (ParseErrorBundle Text CustomErr) String)
 
 -- | A safe version of fixSmartDateStr.
-fixSmartDateStrEither :: Day -> Text -> Either (ParseError Char CustomErr) String
+fixSmartDateStrEither :: Day -> Text -> Either (ParseErrorBundle Text CustomErr) String
 fixSmartDateStrEither d = either Left (Right . showDate) . fixSmartDateStrEither' d
 
-fixSmartDateStrEither' :: Day -> Text -> Either (ParseError Char CustomErr) Day
+fixSmartDateStrEither'
+  :: Day -> Text -> Either (ParseErrorBundle Text CustomErr) Day
 fixSmartDateStrEither' d s = case parsewith smartdateonly (T.toLower s) of
                                Right sd -> Right $ fixSmartDate d sd
                                Left e -> Left e
@@ -526,8 +530,10 @@ prevyear = startofyear . addGregorianYearsClip (-1)
 nextyear = startofyear . addGregorianYearsClip 1
 startofyear day = fromGregorian y 1 1 where (y,_,_) = toGregorian day
 
--- | For given date d find year-long interval that starts on given MM/DD of year
--- and covers it. 
+-- | For given date d find year-long interval that starts on given
+-- MM/DD of year and covers it.
+-- The given MM and DD should be basically valid (1-12 & 1-31),
+-- or an error is raised.
 --
 -- Examples: lets take 2017-11-22. Year-long intervals covering it that
 -- starts before Nov 22 will start in 2017. However
@@ -545,14 +551,19 @@ startofyear day = fromGregorian y 1 1 where (y,_,_) = toGregorian day
 -- 2016-12-31          
 -- >>> nthdayofyearcontaining 1 1 wed22nd
 -- 2017-01-01          
-nthdayofyearcontaining m n d | mmddOfSameYear <= d = mmddOfSameYear
-                             | otherwise = mmddOfPrevYear
-    where mmddOfSameYear = addDays (fromIntegral n-1) $ applyN (m-1) nextmonth s
-          mmddOfPrevYear = addDays (fromIntegral n-1) $ applyN (m-1) nextmonth $ prevyear s
-          s = startofyear d
+nthdayofyearcontaining :: Month -> MonthDay -> Day -> Day
+nthdayofyearcontaining m md date
+  | not (validMonth $ show m)  = error' $ "nthdayofyearcontaining: invalid month "++show m
+  | not (validDay   $ show md) = error' $ "nthdayofyearcontaining: invalid day "  ++show md
+  | mmddOfSameYear <= date = mmddOfSameYear
+  | otherwise = mmddOfPrevYear
+  where mmddOfSameYear = addDays (fromIntegral md-1) $ applyN (m-1) nextmonth s
+        mmddOfPrevYear = addDays (fromIntegral md-1) $ applyN (m-1) nextmonth $ prevyear s
+        s = startofyear date
 
 -- | For given date d find month-long interval that starts on nth day of month
 -- and covers it. 
+-- The given day of month should be basically valid (1-31), or an error is raised.
 --
 -- Examples: lets take 2017-11-22. Month-long intervals covering it that
 -- start on 1st-22nd of month will start in Nov. However
@@ -568,11 +579,14 @@ nthdayofyearcontaining m n d | mmddOfSameYear <= d = mmddOfSameYear
 -- 2017-10-23          
 -- >>> nthdayofmonthcontaining 30 wed22nd
 -- 2017-10-30          
-nthdayofmonthcontaining n d | nthOfSameMonth <= d = nthOfSameMonth
-                            | otherwise = nthOfPrevMonth
-    where nthOfSameMonth = nthdayofmonth n s
-          nthOfPrevMonth = nthdayofmonth n $ prevmonth s
-          s = startofmonth d
+nthdayofmonthcontaining :: MonthDay -> Day -> Day
+nthdayofmonthcontaining md date
+  | not (validDay $ show md) = error' $ "nthdayofmonthcontaining: invalid day "  ++show md
+  | nthOfSameMonth <= date = nthOfSameMonth
+  | otherwise = nthOfPrevMonth
+  where nthOfSameMonth = nthdayofmonth md s
+        nthOfPrevMonth = nthdayofmonth md $ prevmonth s
+        s = startofmonth date
 
 -- | For given date d find week-long interval that starts on nth day of week
 -- and covers it. 
@@ -591,6 +605,7 @@ nthdayofmonthcontaining n d | nthOfSameMonth <= d = nthOfSameMonth
 -- 2017-11-16          
 -- >>> nthdayofweekcontaining 5 wed22nd
 -- 2017-11-17          
+nthdayofweekcontaining :: WeekDay -> Day -> Day
 nthdayofweekcontaining n d | nthOfSameWeek <= d = nthOfSameWeek
                            | otherwise = nthOfPrevWeek
     where nthOfSameWeek = addDays (fromIntegral n-1) s
@@ -614,16 +629,20 @@ nthdayofweekcontaining n d | nthOfSameWeek <= d = nthOfSameWeek
 -- 2017-10-26
 -- >>> nthweekdayofmonthcontaining 4 5 wed22nd
 -- 2017-10-27
+nthweekdayofmonthcontaining :: Int -> WeekDay -> Day -> Day
 nthweekdayofmonthcontaining n wd d | nthWeekdaySameMonth <= d  = nthWeekdaySameMonth
                                    | otherwise = nthWeekdayPrevMonth
     where nthWeekdaySameMonth = advancetonthweekday n wd $ startofmonth d
           nthWeekdayPrevMonth = advancetonthweekday n wd $ prevmonth d
-          
+
 -- | Advance to nth weekday wd after given start day s
-advancetonthweekday n wd s = addWeeks (n-1) . firstMatch (>=s) . iterate (addWeeks 1) $ firstweekday s
-  where          
+advancetonthweekday :: Int -> WeekDay -> Day -> Day
+advancetonthweekday n wd s = 
+  maybe err (addWeeks (n-1)) $ firstMatch (>=s) $ iterate (addWeeks 1) $ firstweekday s
+  where
+    err = error' "advancetonthweekday: should not happen"
     addWeeks k = addDays (7 * fromIntegral k)
-    firstMatch p = head . dropWhile (not . p)
+    firstMatch p = headMay . dropWhile (not . p) 
     firstweekday = addDays (fromIntegral wd-1) . startofweek
 
 ----------------------------------------------------------------------
@@ -822,14 +841,18 @@ md = do
   failIfInvalidDay d
   return ("",m,d)
 
+-- These are compared case insensitively, and should all be kept lower case. 
 months         = ["january","february","march","april","may","june",
                   "july","august","september","october","november","december"]
 monthabbrevs   = ["jan","feb","mar","apr","may","jun","jul","aug","sep","oct","nov","dec"]
 weekdays       = ["monday","tuesday","wednesday","thursday","friday","saturday","sunday"]
 weekdayabbrevs = ["mon","tue","wed","thu","fri","sat","sun"]
 
-monthIndex t = maybe 0 (+1) $ t `elemIndex` months
-monIndex t   = maybe 0 (+1) $ t `elemIndex` monthabbrevs
+-- | Convert a case insensitive english month name to a month number.
+monthIndex name = maybe 0 (+1) $ T.toLower name `elemIndex` months
+
+-- | Convert a case insensitive english three-letter month abbreviation to a month number.
+monIndex   name = maybe 0 (+1) $ T.toLower name `elemIndex` monthabbrevs
 
 month :: TextParser m SmartDate
 month = do
@@ -845,9 +868,11 @@ mon = do
 
 weekday :: TextParser m Int
 weekday = do
-  wday <- choice . map string' $ weekdays ++ weekdayabbrevs
-  let i = head . catMaybes $ [wday `elemIndex` weekdays, wday `elemIndex` weekdayabbrevs]
-  return (i+1)
+  wday <- T.toLower <$> (choice . map string' $ weekdays ++ weekdayabbrevs)
+  case catMaybes $ [wday `elemIndex` weekdays, wday `elemIndex` weekdayabbrevs] of
+    (i:_) -> return (i+1)
+    []    -> fail  $ "weekday: should not happen: attempted to find " <> 
+                      show wday <> " in " <> show (weekdays ++ weekdayabbrevs) 
 
 today,yesterday,tomorrow :: TextParser m SmartDate
 today     = string' "today"     >> return ("","","today")
@@ -966,7 +991,9 @@ reportingintervalp = choice' [
                           return $ DayOfMonth n,
                        do string' "every"
                           let mnth = choice' [month, mon] >>= \(_,m,_) -> return (read m)
-                          d_o_y <- makePermParser $ DayOfYear <$$> try (skipMany spacenonewline *> mnth) <||> try (skipMany spacenonewline *> nth)
+                          d_o_y <- runPermutation $
+                            DayOfYear <$> toPermutation (try (skipMany spacenonewline *> mnth))
+                                      <*> toPermutation (try (skipMany spacenonewline *> nth))
                           optOf_ "year"
                           return d_o_y,
                        do string' "every"
@@ -1071,5 +1098,9 @@ mkdatespan b = DateSpan (Just $ parsedate b) . Just . parsedate
 nulldatespan :: DateSpan
 nulldatespan = DateSpan Nothing Nothing
 
+-- | A datespan of zero length, that matches no date.
+emptydatespan :: DateSpan
+emptydatespan = DateSpan (Just $ addDays 1 nulldate) (Just nulldate)
+
 nulldate :: Day
 nulldate = fromGregorian 0 1 1
