@@ -13,7 +13,7 @@ other data format (see "Hledger.Read").
 module Hledger.Data.Journal (
   -- * Parsing helpers
   addMarketPrice,
-  addModifierTransaction,
+  addTransactionModifier,
   addPeriodicTransaction,
   addTransaction,
   journalBalanceTransactions,
@@ -21,7 +21,8 @@ module Hledger.Data.Journal (
   commodityStylesFromAmounts,
   journalCommodityStyles,
   journalConvertAmountsToCost,
-  journalFinalise,
+  journalReverse,
+  journalSetLastReadTime,
   journalPivot,
   -- * Filtering
   filterJournalTransactions,
@@ -53,7 +54,7 @@ module Hledger.Data.Journal (
   -- * Standard account types
   journalBalanceSheetAccountQuery,
   journalProfitAndLossAccountQuery,
-  journalIncomeAccountQuery,
+  journalRevenueAccountQuery,
   journalExpenseAccountQuery,
   journalAssetAccountQuery,
   journalLiabilityAccountQuery,
@@ -68,7 +69,7 @@ module Hledger.Data.Journal (
   journalUntieTransactions,
   -- * Tests
   samplejournal,
-  tests_Hledger_Data_Journal,
+  tests_Journal,
 )
 where
 import Control.Applicative (Const(..))
@@ -82,7 +83,6 @@ import Data.Functor.Identity (Identity(..))
 import qualified Data.HashTable.ST.Cuckoo as HT
 import Data.List
 import Data.List.Extra (groupSort)
--- import Data.Map (findWithDefault)
 import Data.Maybe
 #if !(MIN_VERSION_base(4,11,0))
 import Data.Monoid
@@ -95,15 +95,14 @@ import Safe (headMay, headDef)
 import Data.Time.Calendar
 import Data.Tree
 import System.Time (ClockTime(TOD))
-import Test.HUnit
 import Text.Printf
 import qualified Data.Map as M
+import qualified Data.Set as S
 
-import Hledger.Utils
+import Hledger.Utils 
 import Hledger.Data.Types
 import Hledger.Data.AccountName
 import Hledger.Data.Amount
--- import Hledger.Data.Commodity
 import Hledger.Data.Dates
 import Hledger.Data.Transaction
 import Hledger.Data.Posting
@@ -138,7 +137,7 @@ instance Show Journal where
 -- showJournalDebug j = unlines [
 --                       show j
 --                      ,show (jtxns j)
---                      ,show (jmodifiertxns j)
+--                      ,show (jtxnmodifiers j)
 --                      ,show (jperiodictxns j)
 --                      ,show $ jparsetimeclockentries j
 --                      ,show $ jmarketprices j
@@ -165,15 +164,17 @@ instance Sem.Semigroup Journal where
     ,jparseparentaccounts       = jparseparentaccounts       j2
     ,jparsealiases              = jparsealiases              j2
     -- ,jparsetransactioncount     = jparsetransactioncount     j1 +  jparsetransactioncount     j2
-    ,jparsetimeclockentries = jparsetimeclockentries j1 <> jparsetimeclockentries j2
-    ,jaccounts                  = jaccounts                  j1 <> jaccounts                  j2
+    ,jparsetimeclockentries     = jparsetimeclockentries j1 <> jparsetimeclockentries j2
+    ,jincludefilestack          = jincludefilestack          j2
+    ,jdeclaredaccounts          = jdeclaredaccounts          j1 <> jdeclaredaccounts          j2
+    ,jdeclaredaccounttypes      = jdeclaredaccounttypes      j1 <> jdeclaredaccounttypes      j2
     ,jcommodities               = jcommodities               j1 <> jcommodities               j2
     ,jinferredcommodities       = jinferredcommodities       j1 <> jinferredcommodities       j2
     ,jmarketprices              = jmarketprices              j1 <> jmarketprices              j2
-    ,jmodifiertxns              = jmodifiertxns              j1 <> jmodifiertxns              j2
+    ,jtxnmodifiers              = jtxnmodifiers              j1 <> jtxnmodifiers              j2
     ,jperiodictxns              = jperiodictxns              j1 <> jperiodictxns              j2
     ,jtxns                      = jtxns                      j1 <> jtxns                      j2
-    ,jfinalcommentlines         = jfinalcommentlines         j2
+    ,jfinalcommentlines         = jfinalcommentlines         j2  -- XXX discards j1's ?
     ,jfiles                     = jfiles                     j1 <> jfiles                     j2
     ,jlastreadtime              = max (jlastreadtime j1) (jlastreadtime j2)
     }
@@ -192,12 +193,14 @@ nulljournal = Journal {
   ,jparseparentaccounts       = []
   ,jparsealiases              = []
   -- ,jparsetransactioncount     = 0
-  ,jparsetimeclockentries = []
-  ,jaccounts                  = []
-  ,jcommodities               = M.fromList []
-  ,jinferredcommodities       = M.fromList []
+  ,jparsetimeclockentries     = []
+  ,jincludefilestack          = []
+  ,jdeclaredaccounts          = []
+  ,jdeclaredaccounttypes      = M.empty
+  ,jcommodities               = M.empty
+  ,jinferredcommodities       = M.empty
   ,jmarketprices              = []
-  ,jmodifiertxns              = []
+  ,jtxnmodifiers              = []
   ,jperiodictxns              = []
   ,jtxns                      = []
   ,jfinalcommentlines         = ""
@@ -217,8 +220,8 @@ mainfile = headDef ("", "") . jfiles
 addTransaction :: Transaction -> Journal -> Journal
 addTransaction t j = j { jtxns = t : jtxns j }
 
-addModifierTransaction :: ModifierTransaction -> Journal -> Journal
-addModifierTransaction mt j = j { jmodifiertxns = mt : jmodifiertxns j }
+addTransactionModifier :: TransactionModifier -> Journal -> Journal
+addTransactionModifier mt j = j { jtxnmodifiers = mt : jtxnmodifiers j }
 
 addPeriodicTransaction :: PeriodicTransaction -> Journal -> Journal
 addPeriodicTransaction pt j = j { jperiodictxns = pt : jperiodictxns j }
@@ -259,7 +262,7 @@ journalAccountNamesImplied = expandAccountNames . journalAccountNamesUsed
 
 -- | Sorted unique account names declared by account directives in this journal.
 journalAccountNamesDeclared :: Journal -> [AccountName]
-journalAccountNamesDeclared = nub . sort . map fst . jaccounts
+journalAccountNamesDeclared = nub . sort . jdeclaredaccounts
 
 -- | Sorted unique account names declared by account directives or posted to
 -- by transactions in this journal.
@@ -278,52 +281,85 @@ journalAccountNames = journalAccountNamesDeclaredOrImplied
 journalAccountNameTree :: Journal -> Tree AccountName
 journalAccountNameTree = accountNameTreeFrom . journalAccountNames
 
--- standard account types
+-- queries for standard account types
 
--- | A query for Profit & Loss accounts in this journal.
--- Cf <http://en.wikipedia.org/wiki/Chart_of_accounts#Profit_.26_Loss_accounts>.
-journalProfitAndLossAccountQuery  :: Journal -> Query
-journalProfitAndLossAccountQuery j = Or [journalIncomeAccountQuery j
-                                               ,journalExpenseAccountQuery j
-                                               ]
+-- | Get a query for accounts of a certain type (Asset, Liability..) in this journal.  
+-- The query will match all accounts which were declared as that type by account directives, 
+-- plus all their subaccounts which have not been declared as a different type. 
+-- If no accounts were declared as this type, the query will instead match accounts 
+-- with names matched by the provided case-insensitive regular expression.
+journalAccountTypeQuery :: AccountType -> Regexp -> Journal -> Query
+journalAccountTypeQuery atype fallbackregex j =
+  case M.lookup atype (jdeclaredaccounttypes j) of
+    Nothing -> Acct fallbackregex
+    Just as ->
+      -- XXX Query isn't able to match account type since that requires extra info from the journal. 
+      -- So we do a hacky search by name instead.
+      And [ 
+         Or $ map (Acct . accountNameToAccountRegex) as
+        ,Not $ Or $ map (Acct . accountNameToAccountRegex) differentlytypedsubs
+        ]
+      where
+        differentlytypedsubs = concat 
+          [subs | (t,bs) <- M.toList (jdeclaredaccounttypes j)
+              , t /= atype
+              , let subs = [b | b <- bs, any (`isAccountNamePrefixOf` b) as]
+          ]
 
--- | A query for Income (Revenue) accounts in this journal.
--- This is currently hard-coded to the case-insensitive regex @^(income|revenue)s?(:|$)@.
-journalIncomeAccountQuery  :: Journal -> Query
-journalIncomeAccountQuery _ = Acct "^(income|revenue)s?(:|$)"
+-- | A query for accounts in this journal which have been
+-- declared as Asset by account directives, or otherwise for
+-- accounts with names matched by the case-insensitive regular expression 
+-- @^assets?(:|$)@.
+journalAssetAccountQuery :: Journal -> Query
+journalAssetAccountQuery = journalAccountTypeQuery Asset "^assets?(:|$)"
 
--- | A query for Expense accounts in this journal.
--- This is currently hard-coded to the case-insensitive regex @^expenses?(:|$)@.
+-- | A query for accounts in this journal which have been
+-- declared as Liability by account directives, or otherwise for
+-- accounts with names matched by the case-insensitive regular expression 
+-- @^(debts?|liabilit(y|ies))(:|$)@.
+journalLiabilityAccountQuery :: Journal -> Query
+journalLiabilityAccountQuery = journalAccountTypeQuery Liability "^(debts?|liabilit(y|ies))(:|$)"
+
+-- | A query for accounts in this journal which have been
+-- declared as Equity by account directives, or otherwise for
+-- accounts with names matched by the case-insensitive regular expression 
+-- @^equity(:|$)@.
+journalEquityAccountQuery :: Journal -> Query
+journalEquityAccountQuery = journalAccountTypeQuery Equity "^equity(:|$)"
+
+-- | A query for accounts in this journal which have been
+-- declared as Revenue by account directives, or otherwise for
+-- accounts with names matched by the case-insensitive regular expression 
+-- @^(income|revenue)s?(:|$)@.
+journalRevenueAccountQuery :: Journal -> Query
+journalRevenueAccountQuery = journalAccountTypeQuery Revenue "^(income|revenue)s?(:|$)"
+
+-- | A query for accounts in this journal which have been
+-- declared as Expense by account directives, or otherwise for
+-- accounts with names matched by the case-insensitive regular expression 
+-- @^(income|revenue)s?(:|$)@.
 journalExpenseAccountQuery  :: Journal -> Query
-journalExpenseAccountQuery _ = Acct "^expenses?(:|$)"
+journalExpenseAccountQuery = journalAccountTypeQuery Expense "^expenses?(:|$)"
 
 -- | A query for Asset, Liability & Equity accounts in this journal.
 -- Cf <http://en.wikipedia.org/wiki/Chart_of_accounts#Balance_Sheet_Accounts>.
 journalBalanceSheetAccountQuery  :: Journal -> Query
 journalBalanceSheetAccountQuery j = Or [journalAssetAccountQuery j
-                                              ,journalLiabilityAccountQuery j
-                                              ,journalEquityAccountQuery j
-                                              ]
+                                       ,journalLiabilityAccountQuery j
+                                       ,journalEquityAccountQuery j
+                                       ]
 
--- | A query for Asset accounts in this journal.
--- This is currently hard-coded to the case-insensitive regex @^assets?(:|$)@.
-journalAssetAccountQuery  :: Journal -> Query
-journalAssetAccountQuery _ = Acct "^assets?(:|$)"
-
--- | A query for Liability accounts in this journal.
--- This is currently hard-coded to the case-insensitive regex @^(debts?|liabilit(y|ies))(:|$)@.
-journalLiabilityAccountQuery  :: Journal -> Query
-journalLiabilityAccountQuery _ = Acct "^(debts?|liabilit(y|ies))(:|$)"
-
--- | A query for Equity accounts in this journal.
--- This is currently hard-coded to the case-insensitive regex @^equity(:|$)@.
-journalEquityAccountQuery  :: Journal -> Query
-journalEquityAccountQuery _ = Acct "^equity(:|$)"
+-- | A query for Profit & Loss accounts in this journal.
+-- Cf <http://en.wikipedia.org/wiki/Chart_of_accounts#Profit_.26_Loss_accounts>.
+journalProfitAndLossAccountQuery  :: Journal -> Query
+journalProfitAndLossAccountQuery j = Or [journalRevenueAccountQuery j
+                                        ,journalExpenseAccountQuery j
+                                        ]
 
 -- | A query for Cash (-equivalent) accounts in this journal (ie,
 -- accounts which appear on the cashflow statement.)  This is currently
--- hard-coded to be all the Asset accounts except for those containing the
--- case-insensitive regex @(receivable|:A/R|:fixed)@.
+-- hard-coded to be all the Asset accounts except for those with names 
+-- containing the case-insensitive regular expression @(receivable|:A/R|:fixed)@.
 journalCashAccountQuery  :: Journal -> Query
 journalCashAccountQuery j = And [journalAssetAccountQuery j, Not $ Acct "(receivable|:A/R|:fixed)"]
 
@@ -485,22 +521,23 @@ filterJournalTransactionsByAccount apats j@Journal{jtxns=ts} = j{jtxns=filter tm
 
 -}
 
--- | Do post-parse processing on a parsed journal to make it ready for
--- use.  Reverse parsed data to normal order, standardise amount
--- formats, check/ensure that transactions are balanced, and maybe
--- check balance assertions.
-journalFinalise :: ClockTime -> FilePath -> Text -> Bool -> ParsedJournal -> Either String Journal
-journalFinalise t path txt assrt j@Journal{jfiles=fs} =
-  journalTieTransactions <$> 
-  (journalBalanceTransactions assrt $
-   journalApplyCommodityStyles $
-   j {jfiles        = (path,txt) : reverse fs
-     ,jlastreadtime = t
-     ,jtxns         = reverse $ jtxns j -- NOTE: see addTransaction
-     ,jmodifiertxns = reverse $ jmodifiertxns j -- NOTE: see addModifierTransaction
-     ,jperiodictxns = reverse $ jperiodictxns j -- NOTE: see addPeriodicTransaction
-     ,jmarketprices = reverse $ jmarketprices j -- NOTE: see addMarketPrice
-     })
+-- | Reverse parsed data to normal order. This is used for post-parse
+-- processing, since data is added to the head of the list during
+-- parsing.
+journalReverse :: Journal -> Journal
+journalReverse j =
+  j {jfiles            = reverse $ jfiles j
+    ,jdeclaredaccounts = reverse $ jdeclaredaccounts j
+    ,jtxns             = reverse $ jtxns j
+    ,jtxnmodifiers     = reverse $ jtxnmodifiers j
+    ,jperiodictxns     = reverse $ jperiodictxns j
+    ,jmarketprices     = reverse $ jmarketprices j
+    }
+
+-- | Set this journal's last read time, ie when its files were last read.
+journalSetLastReadTime :: ClockTime -> Journal -> Journal
+journalSetLastReadTime t j = j{ jlastreadtime = t }
+
 
 journalNumberAndTieTransactions = journalTieTransactions . journalNumberTransactions
 
@@ -532,12 +569,24 @@ journalCheckBalanceAssertions j =
 -- | Check a posting's balance assertion and return an error if it
 -- fails.
 checkBalanceAssertion :: Posting -> MixedAmount -> Either String ()
-checkBalanceAssertion p@Posting{ pbalanceassertion = Just (ass,_)} amt
+checkBalanceAssertion p@Posting{ pbalanceassertion = Just ass } bal =
+  foldl' fold (Right ()) amts
+    where fold (Right _) cass = checkBalanceAssertionCommodity p cass bal
+          fold err _ = err
+          amt = baamount ass
+          amts = amt : if baexact ass
+            then map (\a -> a{ aquantity = 0 }) $ amounts $ filterMixedAmount (\a -> acommodity a /= assertedcomm) bal
+            else []
+          assertedcomm = acommodity amt
+checkBalanceAssertion _ _ = Right ()
+
+checkBalanceAssertionCommodity :: Posting -> Amount -> MixedAmount -> Either String ()
+checkBalanceAssertionCommodity p amt bal
   | isReallyZeroAmount diff = Right ()
   | True    = Left err
-    where assertedcomm = acommodity ass
-          actualbal = fromMaybe nullamt $ find ((== assertedcomm) . acommodity) (amounts amt)
-          diff = ass - actualbal
+    where assertedcomm = acommodity amt
+          actualbal = fromMaybe nullamt $ find ((== assertedcomm) . acommodity) (amounts bal)
+          diff = amt - actualbal
           diffplus | isNegativeAmount diff == False = "+"
                    | otherwise = ""
           err = printf (unlines
@@ -554,16 +603,15 @@ checkBalanceAssertion p@Posting{ pbalanceassertion = Just (ass,_)} amt
             (case ptransaction p of
                Nothing -> ":" -- shouldn't happen
                Just t ->  printf " in %s:\nin transaction:\n%s"
-                          (showGenericSourcePos pos) (chomp $ show t) :: String
-                            where pos = snd $ fromJust $ pbalanceassertion p)
+                          (showGenericSourcePos pos) (chomp $ showTransaction t) :: String
+                            where pos = baposition $ fromJust $ pbalanceassertion p)
             (showPostingLine p)
             (showDate $ postingDate p)
             (T.unpack $ paccount p) -- XXX pack
             assertedcomm
             (showAmount actualbal)
-            (showAmount ass)
+            (showAmount amt)
             (diffplus ++ showAmount diff)
-checkBalanceAssertion _ _ = Right ()
 
 -- | Fill in any missing amounts and check that all journal transactions
 -- balance, or return an error message. This is done after parsing all
@@ -594,6 +642,7 @@ journalBalanceTransactionsST assrt j createStore storeIn extract =
                   (storeIn txStore) 
                   assrt
                   (Just $ journalCommodityStyles j)
+                  (getModifierAccountNames j)
     flip R.runReaderT env $ do
       dated <- fmap snd . sortBy (comparing fst) . concat
                 <$> mapM' discriminateByDate (jtxns j)
@@ -602,16 +651,25 @@ journalBalanceTransactionsST assrt j createStore storeIn extract =
     where 
       size = genericLength $ journalPostings j
 
+
+-- | Collect account names in account modifiers into a set
+getModifierAccountNames :: Journal -> S.Set AccountName
+getModifierAccountNames j = S.fromList $
+                            map paccount $
+                            concatMap tmpostingrules $
+                            jtxnmodifiers j
+
 -- | Monad transformer stack with a reference to a mutable hashtable
 -- of current account balances and a mutable array of finished
 -- transactions in original parsing order.
 type CurrentBalancesModifier s = R.ReaderT (Env s) (ExceptT String (ST s))
 
 -- | Environment for 'CurrentBalancesModifier'
-data Env s = Env { eBalances :: HT.HashTable s AccountName MixedAmount
-                 , eStoreTx  :: Transaction -> ST s ()
-                 , eAssrt    :: Bool
-                 , eStyles   :: Maybe (M.Map CommoditySymbol AmountStyle) 
+data Env s = Env { eBalances     :: HT.HashTable s AccountName MixedAmount
+                 , eStoreTx      :: Transaction -> ST s ()
+                 , eAssrt        :: Bool
+                 , eStyles       :: Maybe (M.Map CommoditySymbol AmountStyle)
+                 , eUnassignable :: S.Set AccountName
                  }
 
 -- | This converts a transaction into a list of transactions or
@@ -636,12 +694,27 @@ discriminateByDate tx
   | True                         = do
       when (any (isJust . pdate) $ tpostings tx) $
         throwError $ unlines $
-        ["Not supported: Transactions with balance assignments "
-        ,"AND dated postings without amount:\n"
+        ["postings may not have both a custom date and a balance assignment."
+        ,"Write the posting amount explicitly, or remove the posting date:\n"
         , showTransaction tx]
       return 
         [(tdate tx, Right $ tx { tpostings = removePrices <$> tpostings tx })]
 
+-- | Throw an error if a posting is in the unassignable set.
+checkUnassignablePosting :: Posting -> CurrentBalancesModifier s ()
+checkUnassignablePosting p = do
+  unassignable <- R.asks eUnassignable
+  if (isAssignment p && paccount p `S.member` unassignable)
+    then throwError $ unlines $
+         [ "cannot assign amount to account "
+         , ""
+         , "    " ++ (T.unpack $ paccount p)
+         , ""
+         , "because it is also included in transaction modifiers."
+         ]
+    else return ()
+
+
 -- | This function takes an object describing changes to
 -- account balances on a single day - either a single posting 
 -- (from an already balanced transaction without assignments)
@@ -670,19 +743,36 @@ discriminateByDate tx
 -- and then balance and store the transaction.
 checkInferAndRegisterAmounts :: Either Posting Transaction
                              -> CurrentBalancesModifier s ()
-checkInferAndRegisterAmounts (Left p) =
+checkInferAndRegisterAmounts (Left p) = do
+  checkUnassignablePosting p
   void $ addAmountAndCheckBalance return p
 checkInferAndRegisterAmounts (Right oldTx) = do
   let ps = tpostings oldTx
+  mapM_ checkUnassignablePosting ps
   styles <- R.reader $ eStyles
   newPostings <- forM ps $ addAmountAndCheckBalance inferFromAssignment
   storeTransaction =<< balanceTransactionUpdate
     (fmap void . addToBalance) styles oldTx { tpostings = newPostings }
   where
     inferFromAssignment :: Posting -> CurrentBalancesModifier s Posting
-    inferFromAssignment p = maybe (return p)
-      (fmap (\a -> p { pamount = a, porigin = Just $ originalPosting p }) . setBalance (paccount p) . fst)
-      $ pbalanceassertion p
+    inferFromAssignment p = do
+      let acc = paccount p
+      case pbalanceassertion p of
+        Just ba | baexact ba -> do
+          diff <- setMixedBalance acc $ Mixed [baamount ba]
+          fullPosting diff p
+        Just ba | otherwise -> do
+          old <- liftModifier $ \Env{ eBalances = bals } -> HT.lookup bals acc
+          let amt = baamount ba
+              assertedcomm = acommodity amt
+          diff <- setMixedBalance acc $
+            Mixed [amt] + filterMixedAmount (\a -> acommodity a /= assertedcomm) (fromMaybe nullmixedamt old)
+          fullPosting diff p
+        Nothing -> return p
+    fullPosting amt p = return p
+      { pamount = amt
+      , porigin = Just $ originalPosting p
+      }
 
 -- | Adds a posting's amount to the posting's account balance and
 -- checks a possible balance assertion. Or if there is no amount,
@@ -698,15 +788,13 @@ addAmountAndCheckBalance _ p | hasAmount p = do
   return p
 addAmountAndCheckBalance fallback p = fallback p
 
--- | Sets an account's balance to a given amount and returns the
--- difference of new and old amount.
-setBalance :: AccountName -> Amount -> CurrentBalancesModifier s MixedAmount
-setBalance acc amt = liftModifier $ \Env{ eBalances = bals } -> do
+-- | Sets all commodities comprising an account's balance to the given
+-- amounts and returns the difference from the previous balance.
+setMixedBalance :: AccountName -> MixedAmount -> CurrentBalancesModifier s MixedAmount
+setMixedBalance acc amt = liftModifier $ \Env{ eBalances = bals } -> do
   old <- HT.lookup bals acc
-  let new = Mixed $ (amt :) $ maybe []
-              (filter ((/= acommodity amt) . acommodity) . amounts) old
-  HT.insert bals acc new
-  return $ maybe new (new -) old
+  HT.insert bals acc amt
+  return $ maybe amt (amt -) old
 
 -- | Adds an amount to an account's balance and returns the resulting balance.
 addToBalance :: AccountName -> MixedAmount -> CurrentBalancesModifier s MixedAmount
@@ -765,8 +853,7 @@ commodityStylesFromAmounts amts = M.fromList commstyles
     commstyles = [(c, canonicalStyleFrom $ map astyle as) | (c,as) <- commamts]
 
 -- | Given an ordered list of amount styles, choose a canonical style.
--- That is: the style of the first, and the
--- maximum precision of all.
+-- That is: the style of the first, and the maximum precision of all.
 canonicalStyleFrom :: [AmountStyle] -> AmountStyle
 canonicalStyleFrom [] = amountstyle
 canonicalStyleFrom ss@(first:_) =
@@ -881,21 +968,6 @@ journalDateSpan secondary j
       pdates   = concatMap (catMaybes . map (if secondary then (Just . postingDate2) else pdate) . tpostings) ts
       ts       = jtxns j
 
--- #ifdef TESTS
-test_journalDateSpan = do
- "journalDateSpan" ~: do
-  assertEqual "" (DateSpan (Just $ fromGregorian 2014 1 10) (Just $ fromGregorian 2014 10 11))
-                 (journalDateSpan True j)
-  where
-    j = nulljournal{jtxns = [nulltransaction{tdate = parsedate "2014/02/01"
-                                            ,tpostings = [posting{pdate=Just (parsedate "2014/01/10")}]
-                                            }
-                            ,nulltransaction{tdate = parsedate "2014/09/01"
-                                            ,tpostings = [posting{pdate2=Just (parsedate "2014/10/10")}]
-                                            }
-                            ]}
--- #endif
-
 -- | Apply the pivot transformation to all postings in a journal,
 -- replacing their account name by their value for the given field or tag.
 journalPivot :: Text -> Journal -> Journal
@@ -1079,12 +1151,32 @@ Right samplejournal = journalBalanceTransactions False $
           ]
          }
 
-tests_Hledger_Data_Journal = TestList $
- [
-  test_journalDateSpan
-  -- "query standard account types" ~:
-  --  do
-  --   let j = journal1
-  --   journalBalanceSheetAccountNames j `is` ["assets","assets:a","equity","equity:q","equity:q:qq","liabilities","liabilities:l"]
-  --   journalProfitAndLossAccountNames j `is` ["expenses","expenses:e","income","income:i"]
- ]
+tests_Journal = tests "Journal" [
+
+   test "journalDateSpan" $
+    journalDateSpan True nulljournal{
+      jtxns = [nulltransaction{tdate = parsedate "2014/02/01"
+                              ,tpostings = [posting{pdate=Just (parsedate "2014/01/10")}]
+                              }
+              ,nulltransaction{tdate = parsedate "2014/09/01"
+                              ,tpostings = [posting{pdate2=Just (parsedate "2014/10/10")}]
+                              }
+              ]
+      }
+    `is` (DateSpan (Just $ fromGregorian 2014 1 10) (Just $ fromGregorian 2014 10 11))
+
+  ,tests "standard account type queries" $
+    let
+      j = samplejournal
+      journalAccountNamesMatching :: Query -> Journal -> [AccountName]
+      journalAccountNamesMatching q = filter (q `matchesAccount`) . journalAccountNames
+      namesfrom qfunc = journalAccountNamesMatching (qfunc j) j
+    in [
+       test "assets"      $ expectEq (namesfrom journalAssetAccountQuery)     ["assets","assets:bank","assets:bank:checking","assets:bank:saving","assets:cash"]
+      ,test "liabilities" $ expectEq (namesfrom journalLiabilityAccountQuery) ["liabilities","liabilities:debts"]
+      ,test "equity"      $ expectEq (namesfrom journalEquityAccountQuery)    []
+      ,test "income"      $ expectEq (namesfrom journalRevenueAccountQuery)    ["income","income:gifts","income:salary"]
+      ,test "expenses"    $ expectEq (namesfrom journalExpenseAccountQuery)   ["expenses","expenses:food","expenses:supplies"]
+    ]
+
+  ]
