@@ -13,7 +13,6 @@ value of things at a given date.
 module Hledger.Data.MarketPrice
 where
 import qualified Data.Text as T
-import Test.HUnit
 
 import Hledger.Data.Amount
 import Hledger.Data.Dates
@@ -28,5 +27,3 @@ showMarketPrice mp = unwords
     , T.unpack (mpcommodity mp)
     , (showAmount . setAmountPrecision maxprecision) (mpamount mp)
     ]
-
-tests_Hledger_Data_MarketPrice = TestList []
