@@ -12,7 +12,6 @@ tags.
 
 module Hledger.Data.Transaction (
   -- * Transaction
-  nullsourcepos,
   nulltransaction,
   txnTieKnot,
   txnUntieKnot,
@@ -42,8 +41,8 @@ module Hledger.Data.Transaction (
   sourceFilePath,
   sourceFirstLine,
   showGenericSourcePos,
-  -- * misc.
-  tests_Hledger_Data_Transaction
+  -- * tests
+  tests_Transaction
 )
 where
 import Data.List
@@ -53,24 +52,15 @@ import Data.Maybe
 import Data.Text (Text)
 import qualified Data.Text as T
 import Data.Time.Calendar
-import Test.HUnit
 import Text.Printf
 import qualified Data.Map as Map
 
-import Hledger.Utils
+import Hledger.Utils 
 import Hledger.Data.Types
 import Hledger.Data.Dates
 import Hledger.Data.Posting
 import Hledger.Data.Amount
 
-instance Show Transaction where show = showTransactionUnelided
-
-instance Show ModifierTransaction where
-    show t = "= " ++ T.unpack (mtvalueexpr t) ++ "\n" ++ unlines (map show (mtpostings t))
-
-instance Show PeriodicTransaction where
-    show t = "~ " ++ T.unpack (ptperiodexpr t) ++ "\n" ++ unlines (map show (ptpostings t))
-
 sourceFilePath :: GenericSourcePos -> FilePath
 sourceFilePath = \case
     GenericSourcePos fp _ _ -> fp
@@ -81,14 +71,13 @@ sourceFirstLine = \case
     GenericSourcePos _ line _ -> line
     JournalSourcePos _ (line, _) -> line
 
+-- | Render source position in human-readable form.
+-- Keep in sync with Hledger.UI.ErrorScreen.hledgerparseerrorpositionp (temporary). XXX
 showGenericSourcePos :: GenericSourcePos -> String
 showGenericSourcePos = \case
     GenericSourcePos fp line column -> show fp ++ " (line " ++ show line ++ ", column " ++ show column ++ ")"
     JournalSourcePos fp (line, line') -> show fp ++ " (lines " ++ show line ++ "-" ++ show line' ++ ")"
 
-nullsourcepos :: GenericSourcePos
-nullsourcepos = GenericSourcePos "" 1 1
-
 nulltransaction :: Transaction
 nulltransaction = Transaction {
                     tindex=0,
@@ -105,8 +94,9 @@ nulltransaction = Transaction {
                   }
 
 {-|
-Show a journal transaction, formatted for the print command. ledger 2.x's
-standard format looks like this:
+Render a journal transaction as text in the style of Ledger's print command. 
+
+Ledger 2.x's standard format looks like this:
 
 @
 yyyy/mm/dd[ *][ CODE] description.........          [  ; comment...............]
@@ -119,51 +109,40 @@ pacctwidth    = 35 minimum, no maximum  -- they were important at the time.
 pamtwidth     = 11
 pcommentwidth = no limit -- 22
 @
+
+The output will be parseable journal syntax.
+To facilitate this, postings with explicit multi-commodity amounts 
+are displayed as multiple similar postings, one per commodity.
+(Normally does not happen with this function).
+
+If there are multiple postings, all with explicit amounts,
+and the transaction appears obviously balanced
+(postings sum to 0, without needing to infer conversion prices),
+the last posting's amount will not be shown.
 -}
+-- XXX why that logic ? 
+-- XXX where is/should this be still used ? 
+-- XXX rename these, after amount expressions/mixed posting amounts lands
+--     eg showTransactionSimpleAmountsElidingLast, showTransactionSimpleAmounts, showTransaction
 showTransaction :: Transaction -> String
 showTransaction = showTransactionHelper True False
 
+-- | Like showTransaction, but does not change amounts' explicitness.
+-- Explicit amounts are shown and implicit amounts are not.
+-- The output will be parseable journal syntax.
+-- To facilitate this, postings with explicit multi-commodity amounts 
+-- are displayed as multiple similar postings, one per commodity.
+-- Most often, this is the one you want to use.
 showTransactionUnelided :: Transaction -> String
 showTransactionUnelided = showTransactionHelper False False
 
-tests_showTransactionUnelided = [
-   "showTransactionUnelided" ~: do
-    let t `gives` s = assertEqual "" s (showTransactionUnelided t)
-    nulltransaction `gives` "0000/01/01\n\n"
-    nulltransaction{
-      tdate=parsedate "2012/05/14",
-      tdate2=Just $ parsedate "2012/05/15",
-      tstatus=Unmarked,
-      tcode="code",
-      tdescription="desc",
-      tcomment="tcomment1\ntcomment2\n",
-      ttags=[("ttag1","val1")],
-      tpostings=[
-        nullposting{
-          pstatus=Cleared,
-          paccount="a",
-          pamount=Mixed [usd 1, hrs 2],
-          pcomment="\npcomment2\n",
-          ptype=RegularPosting,
-          ptags=[("ptag1","val1"),("ptag2","val2")]
-          }
-       ]
-      }
-      `gives` unlines [
-      "2012/05/14=2012/05/15 (code) desc    ; tcomment1",
-      "    ; tcomment2",
-      "    * a         $1.00",
-      "    ; pcomment2",
-      "    * a         2.00h",
-      "    ; pcomment2",
-      ""
-      ]
- ]
-
+-- | Like showTransactionUnelided, but explicit multi-commodity amounts 
+-- are shown on one line, comma-separated. In this case the output will 
+-- not be parseable journal syntax.
 showTransactionUnelidedOneLineAmounts :: Transaction -> String
 showTransactionUnelidedOneLineAmounts = showTransactionHelper False True
 
--- cf showPosting
+-- | Helper for showTransaction*. 
 showTransactionHelper :: Bool -> Bool -> Transaction -> String
 showTransactionHelper elide onelineamounts t =
     unlines $ [descriptionline]
@@ -182,42 +161,72 @@ showTransactionHelper elide onelineamounts t =
         case renderCommentLines (tcomment t) of []   -> ("",[])
                                                 c:cs -> (c,cs)
 
--- Render a transaction or posting's comment as indented, semicolon-prefixed comment lines.
+-- | Render a transaction or posting's comment as indented, semicolon-prefixed comment lines.
 renderCommentLines :: Text -> [String]
 renderCommentLines t  = case lines $ T.unpack t of ("":ls) -> "":map commentprefix ls
                                                    ls      -> map commentprefix ls
     where
       commentprefix = indent . ("; "++)
 
--- -- Render a transaction or posting's comment as semicolon-prefixed comment lines -
--- -- an inline (same-line) comment if it's a single line, otherwise multiple indented lines.
--- commentLines' :: String -> (String, [String])
--- commentLines' s
---     | null s = ("", [])
---     | length ls == 1 = (prefix $ head ls, [])
---     | otherwise = ("", (prefix $ head ls):(map prefix $ tail ls))
---     where
---       ls = lines s
---       prefix = indent . (";"++)
-
+-- | Given a transaction and its postings, render the postings, suitable
+-- for `print` output. Normally this output will be valid journal syntax which
+-- hledger can reparse (though it may include no-longer-valid balance assertions).
+--
+-- Explicit amounts are shown, any implicit amounts are not. 
+--
+-- Setting elide to true forces the last posting's amount to be implicit, if:
+-- there are other postings, all with explicit amounts, and the transaction
+-- appears balanced.
+--
+-- Postings with multicommodity explicit amounts are handled as follows:
+-- if onelineamounts is true, these amounts are shown on one line,
+-- comma-separated, and the output will not be valid journal syntax.
+-- Otherwise, they are shown as several similar postings, one per commodity.
+-- 
+-- The output will appear to be a balanced transaction.
+-- Amounts' display precisions, which may have been limited by commodity
+-- directives, will be increased if necessary to ensure this.
+--
+-- Posting amounts will be aligned with each other, starting about 4 columns
+-- beyond the widest account name (see postingAsLines for details).
+-- 
 postingsAsLines :: Bool -> Bool -> Transaction -> [Posting] -> [String]
 postingsAsLines elide onelineamounts t ps
-    | elide && length ps > 1 && isTransactionBalanced Nothing t -- imprecise balanced check
+    | elide && length ps > 1 && all hasAmount ps && isTransactionBalanced Nothing t -- imprecise balanced check
        = (concatMap (postingAsLines False onelineamounts ps) $ init ps) ++ postingAsLines True onelineamounts ps (last ps)
     | otherwise = concatMap (postingAsLines False onelineamounts ps) ps
 
+-- | Render one posting, on one or more lines, suitable for `print` output.  
+-- There will be an indented account name, plus one or more of status flag,
+-- posting amount, balance assertion, same-line comment, next-line comments.
+-- 
+-- If the posting's amount is implicit or if elideamount is true, no amount is shown.
+--
+-- If the posting's amount is explicit and multi-commodity, multiple similar 
+-- postings are shown, one for each commodity, to help produce parseable journal syntax.
+-- Or if onelineamounts is true, such amounts are shown on one line, comma-separated
+-- (and the output will not be valid journal syntax).
+--
+-- By default, 4 spaces (2 if there's a status flag) are shown between 
+-- account name and start of amount area, which is typically 12 chars wide
+-- and contains a right-aligned amount (so 10-12 visible spaces between 
+-- account name and amount is typical).
+-- When given a list of postings to be aligned with, the whitespace will be 
+-- increased if needed to match the posting with the longest account name.
+-- This is used to align the amounts of a transaction's postings.
+--
 postingAsLines :: Bool -> Bool -> [Posting] -> Posting -> [String]
-postingAsLines elideamount onelineamounts ps p = concat [
+postingAsLines elideamount onelineamounts pstoalignwith p = concat [
     postingblock
     ++ newlinecomments
     | postingblock <- postingblocks]
   where
     postingblocks = [map rstrip $ lines $ concatTopPadded [statusandaccount, "  ", amount, assertion, samelinecomment] | amount <- shownAmounts]
-    assertion = maybe "" ((" = " ++) . showAmountWithZeroCommodity . fst) $ pbalanceassertion p
+    assertion = maybe "" ((" = " ++) . showAmountWithZeroCommodity . baamount) $ pbalanceassertion p
     statusandaccount = indent $ fitString (Just $ minwidth) Nothing False True $ pstatusandacct p
         where
           -- pad to the maximum account name width, plus 2 to leave room for status flags, to keep amounts aligned  
-          minwidth = maximum $ map ((2+) . textWidth . T.pack . pacctstr) ps
+          minwidth = maximum $ map ((2+) . textWidth . T.pack . pacctstr) pstoalignwith
           pstatusandacct p' = pstatusprefix p' ++ pacctstr p'
           pstatusprefix p' | null s    = ""
                            | otherwise = s ++ " "
@@ -231,14 +240,14 @@ postingAsLines elideamount onelineamounts ps p = concat [
       | null (amounts $ pamount p) = [""]
       | otherwise      = map (fitStringMulti (Just amtwidth) Nothing False False . showAmount ) . amounts $ pamount p
       where
-        amtwidth = maximum $ 12 : map (strWidth . showMixedAmount . pamount) ps  -- min. 12 for backwards compatibility
+        amtwidth = maximum $ 12 : map (strWidth . showMixedAmount . pamount) pstoalignwith  -- min. 12 for backwards compatibility
 
     (samelinecomment, newlinecomments) =
       case renderCommentLines (pcomment p) of []   -> ("",[])
                                               c:cs -> (c,cs)
 
-
--- used in balance assertion error
+-- | Show a posting's status, account name and amount on one line. 
+-- Used in balance assertion errors.
 showPostingLine p =
   indent $
   if pstatus p == Cleared then "* " else "" ++
@@ -246,66 +255,13 @@ showPostingLine p =
   "    " ++
   showMixedAmountOneLine (pamount p)
 
--- | Produce posting line with all comment lines associated with it
+-- | Render a posting, at the appropriate width for aligning with
+-- its siblings if any. Used by the rewrite command. 
 showPostingLines :: Posting -> [String]
 showPostingLines p = postingAsLines False False ps p where
     ps | Just t <- ptransaction p = tpostings t
        | otherwise = [p]
 
-tests_postingAsLines = [
-   "postingAsLines" ~: do
-    let p `gives` ls = assertEqual (show p) ls (postingAsLines False False [p] p)
-    posting `gives` [""]
-    posting{
-      pstatus=Cleared,
-      paccount="a",
-      pamount=Mixed [usd 1, hrs 2],
-      pcomment="pcomment1\npcomment2\n  tag3: val3  \n",
-      ptype=RegularPosting,
-      ptags=[("ptag1","val1"),("ptag2","val2")]
-      }
-     `gives` [
-      "    * a         $1.00    ; pcomment1",
-      "    ; pcomment2",
-      "    ;   tag3: val3  ",
-      "    * a         2.00h    ; pcomment1",
-      "    ; pcomment2",
-      "    ;   tag3: val3  "
-      ]
- ]
-
-tests_inference = [
-    "inferBalancingAmount" ~: do
-    let p `gives` p' = assertEqual (show p) (Right p') $ inferTransaction p
-        inferTransaction :: Transaction -> Either String Transaction
-        inferTransaction = runIdentity . runExceptT . inferBalancingAmount (\_ _ -> return ()) Map.empty
-    nulltransaction `gives` nulltransaction
-    nulltransaction{
-      tpostings=[
-        "a" `post` usd (-5),
-        "b" `post` missingamt
-      ]}
-      `gives`
-      nulltransaction{
-        tpostings=[
-          "a" `post` usd (-5),
-          "b" `post` usd 5
-        ]}
-    nulltransaction{
-      tpostings=[
-        "a" `post` usd (-5),
-        "b" `post` (eur 3 @@ usd 4),
-        "c" `post` missingamt
-      ]}
-      `gives`
-      nulltransaction{
-        tpostings=[
-          "a" `post` usd (-5),
-          "b" `post` (eur 3 @@ usd 4),
-          "c" `post` usd 1
-        ]}
- ]
-
 indent :: String -> String
 indent = ("    "++)
 
@@ -349,9 +305,12 @@ transactionPostingBalances t = (sumPostings $ realPostings t
                                ,sumPostings $ virtualPostings t
                                ,sumPostings $ balancedVirtualPostings t)
 
--- | Is this transaction balanced ? A balanced transaction's real
--- (non-virtual) postings sum to 0, and any balanced virtual postings
--- also sum to 0.
+-- | Does this transaction appear balanced when rendered, optionally with the
+-- given commodity display styles ? More precisely:
+-- after converting amounts to cost using explicit transaction prices if any;
+-- and summing the real postings, and summing the balanced virtual postings;
+-- and applying the given display styles if any (maybe affecting decimal places);
+-- do both totals appear to be zero when rendered ?
 isTransactionBalanced :: Maybe (Map.Map CommoditySymbol AmountStyle) -> Transaction -> Bool
 isTransactionBalanced styles t =
     -- isReallyZeroMixedAmountCost rsum && isReallyZeroMixedAmountCost bvsum
@@ -383,13 +342,13 @@ balanceTransactionUpdate :: MonadError String m
   -> Maybe (Map.Map CommoditySymbol AmountStyle)
   -> Transaction -> m Transaction
 balanceTransactionUpdate update mstyles t =
-  finalize =<< inferBalancingAmount update (fromMaybe Map.empty mstyles) t
+  (finalize =<< inferBalancingAmount update (fromMaybe Map.empty mstyles) t)
+    `catchError` (throwError . annotateErrorWithTxn t)
   where
     finalize t' = let t'' = inferBalancingPrices t'
                   in if isTransactionBalanced mstyles t''
                      then return $ txnTieKnot t''
-                     else throwError $ printerr $ nonzerobalanceerror t''
-    printerr s = intercalate "\n" [s, showTransactionUnelided t]
+                     else throwError $ nonzerobalanceerror t''
     nonzerobalanceerror :: Transaction -> String
     nonzerobalanceerror t = printf "could not balance this transaction (%s%s%s)" rmsg sep bvmsg
         where
@@ -402,6 +361,8 @@ balanceTransactionUpdate update mstyles t =
                   ++ showMixedAmount (costOfMixedAmount bvsum)
           sep = if not (null rmsg) && not (null bvmsg) then "; " else "" :: String
 
+    annotateErrorWithTxn t e = intercalate "\n" [showGenericSourcePos $ tsourcepos t, e, showTransactionUnelided t]
+
 -- | Infer up to one missing amount for this transactions's real postings, and
 -- likewise for its balanced virtual postings, if needed; or return an error
 -- message if we can't.
@@ -416,14 +377,13 @@ inferBalancingAmount :: MonadError String m =>
                      -> m Transaction
 inferBalancingAmount update styles t@Transaction{tpostings=ps}
   | length amountlessrealps > 1
-      = throwError $ printerr "could not balance this transaction - can't have more than one real posting with no amount (remember to put 2 or more spaces before amounts)"
+      = throwError "could not balance this transaction - can't have more than one real posting with no amount (remember to put 2 or more spaces before amounts)"
   | length amountlessbvps > 1
-      = throwError $ printerr "could not balance this transaction - can't have more than one balanced virtual posting with no amount (remember to put 2 or more spaces before amounts)"
+      = throwError "could not balance this transaction - can't have more than one balanced virtual posting with no amount (remember to put 2 or more spaces before amounts)"
   | otherwise
       = do postings <- mapM inferamount ps
            return t{tpostings=postings}
   where
-    printerr s = intercalate "\n" [s, showTransactionUnelided t]
     (amountfulrealps, amountlessrealps) = partition hasAmount (realPostings t)
     realsum = sumStrict $ map pamount amountfulrealps
     (amountfulbvps, amountlessbvps) = partition hasAmount (balancedVirtualPostings t)
@@ -514,7 +474,7 @@ priceInferrerFor t pt = inferprice
             fromamount    = head $ filter ((==fromcommodity).acommodity) sumamounts
             tocommodity   = head $ filter (/=fromcommodity) sumcommodities
             toamount      = head $ filter ((==tocommodity).acommodity) sumamounts
-            unitprice     = toamount `divideAmount` (aquantity fromamount)
+            unitprice     = (aquantity fromamount) `divideAmount` toamount
             unitprecision = max 2 ((asprecision $ astyle $ toamount) + (asprecision $ astyle $ fromamount))
     inferprice p = p
 
@@ -537,178 +497,373 @@ txnUntieKnot t@Transaction{tpostings=ps} = t{tpostings=map (\p -> p{ptransaction
 postingSetTransaction :: Transaction -> Posting -> Posting
 postingSetTransaction t p = p{ptransaction=Just t}
 
-tests_Hledger_Data_Transaction = TestList $ concat [
-  tests_postingAsLines,
-  tests_showTransactionUnelided,
-  tests_inference,
-  [
-  "showTransaction" ~: do
-     assertEqual "show a balanced transaction, eliding last amount"
-       (unlines
-        ["2007/01/28 coopportunity"
-        ,"    expenses:food:groceries          $47.18"
-        ,"    assets:checking"
-        ,""
-        ])
+-- tests
+
+tests_Transaction = tests "Transaction" [
+
+  tests "showTransactionUnelided" [
+     showTransactionUnelided nulltransaction `is` "0000/01/01\n\n"
+    ,showTransactionUnelided nulltransaction{
+      tdate=parsedate "2012/05/14",
+      tdate2=Just $ parsedate "2012/05/15",
+      tstatus=Unmarked,
+      tcode="code",
+      tdescription="desc",
+      tcomment="tcomment1\ntcomment2\n",
+      ttags=[("ttag1","val1")],
+      tpostings=[
+        nullposting{
+          pstatus=Cleared,
+          paccount="a",
+          pamount=Mixed [usd 1, hrs 2],
+          pcomment="\npcomment2\n",
+          ptype=RegularPosting,
+          ptags=[("ptag1","val1"),("ptag2","val2")]
+          }
+       ]
+      }
+      `is` unlines [
+      "2012/05/14=2012/05/15 (code) desc    ; tcomment1",
+      "    ; tcomment2",
+      "    * a         $1.00",
+      "    ; pcomment2",
+      "    * a         2.00h",
+      "    ; pcomment2",
+      ""
+      ]
+    ]
+
+  ,tests "postingAsLines" [
+    postingAsLines False False [posting] posting `is` [""]
+    ,let p = posting{
+      pstatus=Cleared,
+      paccount="a",
+      pamount=Mixed [usd 1, hrs 2],
+      pcomment="pcomment1\npcomment2\n  tag3: val3  \n",
+      ptype=RegularPosting,
+      ptags=[("ptag1","val1"),("ptag2","val2")]
+      }
+     in postingAsLines False False [p] p `is`
+      [
+      "    * a         $1.00    ; pcomment1",
+      "    ; pcomment2",
+      "    ;   tag3: val3  ",
+      "    * a         2.00h    ; pcomment1",
+      "    ; pcomment2",
+      "    ;   tag3: val3  "
+      ]
+    ]
+
+   -- postingsAsLines
+  ,let
+    -- one implicit amount 
+    timp = nulltransaction{tpostings=[
+            "a" `post` usd 1,
+            "b" `post` missingamt
+            ]}
+    -- explicit amounts, balanced
+    texp = nulltransaction{tpostings=[
+            "a" `post` usd 1,
+            "b" `post` usd (-1)
+            ]} 
+    -- explicit amount, only one posting
+    texp1 = nulltransaction{tpostings=[
+            "(a)" `post` usd 1
+            ]}
+    -- explicit amounts, two commodities, explicit balancing price
+    texp2 = nulltransaction{tpostings=[
+            "a" `post` usd 1,
+            "b" `post` (hrs (-1) `at` usd 1)
+            ]} 
+    -- explicit amounts, two commodities, implicit balancing price
+    texp2b = nulltransaction{tpostings=[
+            "a" `post` usd 1,
+            "b" `post` hrs (-1)
+            ]}
+    -- one missing amount, not the last one
+    t3 = nulltransaction{tpostings=[
+         "a" `post` usd 1
+        ,"b" `post` missingamt
+        ,"c" `post` usd (-1)
+        ]}
+    -- unbalanced amounts when precision is limited (#931)
+    t4 = nulltransaction{tpostings=[
+         "a" `post` usd (-0.01)
+        ,"b" `post` usd (0.005)
+        ,"c" `post` usd (0.005)
+        ]}
+  in
+    tests "postingsAsLines" [
+
+     test "null-transaction" $
+      let t = nulltransaction
+      in postingsAsLines True False t (tpostings t) `is` []
+
+    ,test "implicit-amount-elide-false" $
+      let t = timp in postingsAsLines False False t (tpostings t) `is` [
+           "    a           $1.00" 
+          ,"    b"                  -- implicit amount remains implicit
+          ]
+
+    ,test "implicit-amount-elide-true" $
+      let t = timp in postingsAsLines True False t (tpostings t) `is` [
+           "    a           $1.00" 
+          ,"    b"                  -- implicit amount remains implicit
+          ]
+
+    ,test "explicit-amounts-elide-false" $
+      let t = texp in postingsAsLines False False t (tpostings t) `is` [
+           "    a           $1.00" 
+          ,"    b          $-1.00"  -- both amounts remain explicit
+          ]
+
+    ,test "explicit-amounts-elide-true" $
+      let t = texp in postingsAsLines True False t (tpostings t) `is` [
+           "    a           $1.00" 
+          ,"    b"                  -- explicit amount is made implicit
+          ]
+
+    ,test "one-explicit-amount-elide-true" $
+      let t = texp1 in postingsAsLines True False t (tpostings t) `is` [
+           "    (a)           $1.00"  -- explicit amount remains explicit since only one posting 
+          ]
+
+    ,test "explicit-amounts-two-commodities-elide-true" $
+      let t = texp2 in postingsAsLines True False t (tpostings t) `is` [
+           "    a             $1.00" 
+          ,"    b"                    -- explicit amount is made implicit since txn is explicitly balanced
+          ]
+
+    ,test "explicit-amounts-not-explicitly-balanced-elide-true" $
+      let t = texp2b in postingsAsLines True False t (tpostings t) `is` [
+           "    a           $1.00" 
+          ,"    b          -1.00h"    -- explicit amount remains explicit since a conversion price would have be inferred to balance
+          ]
+
+    ,test "implicit-amount-not-last" $
+      let t = t3 in postingsAsLines True False t (tpostings t) `is` [
+           "    a           $1.00" 
+          ,"    b"
+          ,"    c          $-1.00"
+          ]
+
+    ,_test "ensure-visibly-balanced" $
+      let t = t4 in postingsAsLines False False t (tpostings t) `is` [
+           "    a          $-0.01"
+          ,"    b           $0.005"
+          ,"    c           $0.005"
+          ]
+
+   ]
+
+  ,do
+    let inferTransaction :: Transaction -> Either String Transaction
+        inferTransaction = runIdentity . runExceptT . inferBalancingAmount (\_ _ -> return ()) Map.empty
+    tests "inferBalancingAmount" [ 
+       inferTransaction nulltransaction `is` Right nulltransaction
+      ,inferTransaction nulltransaction{
+        tpostings=[
+          "a" `post` usd (-5),
+          "b" `post` missingamt
+        ]}
+      `is` Right
+        nulltransaction{
+          tpostings=[
+            "a" `post` usd (-5),
+            "b" `post` usd 5
+          ]}
+      ,inferTransaction nulltransaction{
+        tpostings=[
+          "a" `post` usd (-5),
+          "b" `post` (eur 3 @@ usd 4),
+          "c" `post` missingamt
+        ]}
+      `is` Right
+        nulltransaction{
+          tpostings=[
+            "a" `post` usd (-5),
+            "b" `post` (eur 3 @@ usd 4),
+            "c" `post` usd 1
+          ]}
+      ]
+
+  ,tests "showTransaction" [
+     test "show a balanced transaction, eliding last amount" $
+       let t = Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Unmarked "" "coopportunity" "" []
+                [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18], ptransaction=Just t}
+                ,posting{paccount="assets:checking", pamount=Mixed [usd (-47.18)], ptransaction=Just t}
+                ] ""
+       in 
+        showTransaction t
+         `is`
+         unlines
+          ["2007/01/28 coopportunity"
+          ,"    expenses:food:groceries          $47.18"
+          ,"    assets:checking"
+          ,""
+          ]
+
+    ,test "show a balanced transaction, no eliding" $
        (let t = Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Unmarked "" "coopportunity" "" []
                 [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18], ptransaction=Just t}
                 ,posting{paccount="assets:checking", pamount=Mixed [usd (-47.18)], ptransaction=Just t}
                 ] ""
-        in showTransaction t)
-
-  ,"showTransaction" ~: do
-     assertEqual "show a balanced transaction, no eliding"
+        in showTransactionUnelided t)
+       `is`
        (unlines
         ["2007/01/28 coopportunity"
         ,"    expenses:food:groceries          $47.18"
         ,"    assets:checking                 $-47.18"
         ,""
         ])
-       (let t = Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Unmarked "" "coopportunity" "" []
-                [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18], ptransaction=Just t}
-                ,posting{paccount="assets:checking", pamount=Mixed [usd (-47.18)], ptransaction=Just t}
-                ] ""
-        in showTransactionUnelided t)
 
      -- document some cases that arise in debug/testing:
-  ,"showTransaction" ~: do
-     assertEqual "show an unbalanced transaction, should not elide"
+    ,test "show an unbalanced transaction, should not elide" $
+       (showTransaction
+        (txnTieKnot $ Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Unmarked "" "coopportunity" "" []
+         [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18]}
+         ,posting{paccount="assets:checking", pamount=Mixed [usd (-47.19)]}
+         ] ""))
+       `is`
        (unlines
         ["2007/01/28 coopportunity"
         ,"    expenses:food:groceries          $47.18"
         ,"    assets:checking                 $-47.19"
         ,""
         ])
+
+    ,test "show an unbalanced transaction with one posting, should not elide" $
        (showTransaction
         (txnTieKnot $ Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Unmarked "" "coopportunity" "" []
          [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18]}
-         ,posting{paccount="assets:checking", pamount=Mixed [usd (-47.19)]}
          ] ""))
-
-  ,"showTransaction" ~: do
-     assertEqual "show an unbalanced transaction with one posting, should not elide"
+       `is`
        (unlines
         ["2007/01/28 coopportunity"
         ,"    expenses:food:groceries          $47.18"
         ,""
         ])
+
+    ,test "show a transaction with one posting and a missing amount" $
        (showTransaction
         (txnTieKnot $ Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Unmarked "" "coopportunity" "" []
-         [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18]}
+         [posting{paccount="expenses:food:groceries", pamount=missingmixedamt}
          ] ""))
-
-  ,"showTransaction" ~: do
-     assertEqual "show a transaction with one posting and a missing amount"
+       `is`
        (unlines
         ["2007/01/28 coopportunity"
         ,"    expenses:food:groceries"
         ,""
         ])
-       (showTransaction
-        (txnTieKnot $ Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Unmarked "" "coopportunity" "" []
-         [posting{paccount="expenses:food:groceries", pamount=missingmixedamt}
-         ] ""))
 
-  ,"showTransaction" ~: do
-     assertEqual "show a transaction with a priced commodityless amount"
+    ,test "show a transaction with a priced commodityless amount" $
+       (showTransaction
+        (txnTieKnot $ Transaction 0 nullsourcepos (parsedate "2010/01/01") Nothing Unmarked "" "x" "" []
+         [posting{paccount="a", pamount=Mixed [num 1 `at` (usd 2 `withPrecision` 0)]}
+         ,posting{paccount="b", pamount= missingmixedamt}
+         ] ""))
+       `is`
        (unlines
         ["2010/01/01 x"
         ,"    a          1 @ $2"
         ,"    b"
         ,""
         ])
-       (showTransaction
-        (txnTieKnot $ Transaction 0 nullsourcepos (parsedate "2010/01/01") Nothing Unmarked "" "x" "" []
-         [posting{paccount="a", pamount=Mixed [num 1 `at` (usd 2 `withPrecision` 0)]}
-         ,posting{paccount="b", pamount= missingmixedamt}
-         ] ""))
+    ]
 
-  ,"balanceTransaction" ~: do
-     assertBool "detect unbalanced entry, sign error"
-                    (isLeft $ balanceTransaction Nothing
+  ,tests "balanceTransaction" [
+     test "detect unbalanced entry, sign error" $
+                    (expectLeft $ balanceTransaction Nothing
                            (Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Unmarked "" "test" "" []
                             [posting{paccount="a", pamount=Mixed [usd 1]}
                             ,posting{paccount="b", pamount=Mixed [usd 1]}
                             ] ""))
 
-     assertBool "detect unbalanced entry, multiple missing amounts"
-                    (isLeft $ balanceTransaction Nothing
+    ,test "detect unbalanced entry, multiple missing amounts" $
+                    (expectLeft $ balanceTransaction Nothing
                            (Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Unmarked "" "test" "" []
                             [posting{paccount="a", pamount=missingmixedamt}
                             ,posting{paccount="b", pamount=missingmixedamt}
                             ] ""))
 
-     let e = balanceTransaction Nothing (Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Unmarked "" "" "" []
-                           [posting{paccount="a", pamount=Mixed [usd 1]}
-                           ,posting{paccount="b", pamount=missingmixedamt}
-                           ] "")
-     assertBool "balanceTransaction allows one missing amount" (isRight e)
-     assertEqual "balancing amount is inferred"
-                     (Mixed [usd (-1)])
-                     (case e of
-                        Right e' -> (pamount $ last $ tpostings e')
-                        Left _ -> error' "should not happen")
+    ,test "one missing amount is inferred" $
+         (pamount . last . tpostings <$> balanceTransaction
+           Nothing
+           (Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Unmarked "" "" "" []
+             [posting{paccount="a", pamount=Mixed [usd 1]}
+             ,posting{paccount="b", pamount=missingmixedamt}
+             ] ""))
+         `is` Right (Mixed [usd (-1)])
 
-     let e = balanceTransaction Nothing (Transaction 0 nullsourcepos (parsedate "2011/01/01") Nothing Unmarked "" "" "" []
-                           [posting{paccount="a", pamount=Mixed [usd 1.35]}
-                           ,posting{paccount="b", pamount=Mixed [eur (-1)]}
-                           ] "")
-     assertBool "balanceTransaction can infer conversion price" (isRight e)
-     assertEqual "balancing conversion price is inferred"
-                     (Mixed [usd 1.35 @@ (eur 1 `withPrecision` maxprecision)])
-                     (case e of
-                        Right e' -> (pamount $ head $ tpostings e')
-                        Left _ -> error' "should not happen")
+    ,test "conversion price is inferred" $
+         (pamount . head . tpostings <$> balanceTransaction
+           Nothing
+           (Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Unmarked "" "" "" []
+             [posting{paccount="a", pamount=Mixed [usd 1.35]}
+             ,posting{paccount="b", pamount=Mixed [eur (-1)]}
+             ] ""))
+         `is` Right (Mixed [usd 1.35 @@ (eur 1 `withPrecision` maxprecision)])
 
-     assertBool "balanceTransaction balances based on cost if there are unit prices" (isRight $
+    ,test "balanceTransaction balances based on cost if there are unit prices" $
+       expectRight $
        balanceTransaction Nothing (Transaction 0 nullsourcepos (parsedate "2011/01/01") Nothing Unmarked "" "" "" []
                            [posting{paccount="a", pamount=Mixed [usd 1 `at` eur 2]}
                            ,posting{paccount="a", pamount=Mixed [usd (-2) `at` eur 1]}
-                           ] ""))
+                           ] "")
 
-     assertBool "balanceTransaction balances based on cost if there are total prices" (isRight $
+    ,test "balanceTransaction balances based on cost if there are total prices" $
+       expectRight $
        balanceTransaction Nothing (Transaction 0 nullsourcepos (parsedate "2011/01/01") Nothing Unmarked "" "" "" []
                            [posting{paccount="a", pamount=Mixed [usd 1    @@ eur 1]}
                            ,posting{paccount="a", pamount=Mixed [usd (-2) @@ eur 1]}
-                           ] ""))
+                           ] "")
+  ]
 
-  ,"isTransactionBalanced" ~: do
-     let t = Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Unmarked "" "a" "" []
-             [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
-             ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
+  ,tests "isTransactionBalanced" [
+     test "detect balanced" $ expect $
+       isTransactionBalanced Nothing $ Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Unmarked "" "a" "" []
+             [posting{paccount="b", pamount=Mixed [usd 1.00]}
+             ,posting{paccount="c", pamount=Mixed [usd (-1.00)]}
              ] ""
-     assertBool "detect balanced" (isTransactionBalanced Nothing t)
-     let t = Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Unmarked "" "a" "" []
-             [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
-             ,posting{paccount="c", pamount=Mixed [usd (-1.01)], ptransaction=Just t}
+     
+    ,test "detect unbalanced" $ expect $
+       not $ isTransactionBalanced Nothing $ Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Unmarked "" "a" "" []
+             [posting{paccount="b", pamount=Mixed [usd 1.00]}
+             ,posting{paccount="c", pamount=Mixed [usd (-1.01)]}
              ] ""
-     assertBool "detect unbalanced" (not $ isTransactionBalanced Nothing t)
-     let t = Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Unmarked "" "a" "" []
-             [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
+     
+    ,test "detect unbalanced, one posting" $ expect $
+       not $ isTransactionBalanced Nothing $ Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Unmarked "" "a" "" []
+             [posting{paccount="b", pamount=Mixed [usd 1.00]}
              ] ""
-     assertBool "detect unbalanced, one posting" (not $ isTransactionBalanced Nothing t)
-     let t = Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Unmarked "" "a" "" []
-             [posting{paccount="b", pamount=Mixed [usd 0], ptransaction=Just t}
+     
+    ,test "one zero posting is considered balanced for now" $ expect $
+       isTransactionBalanced Nothing $ Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Unmarked "" "a" "" []
+             [posting{paccount="b", pamount=Mixed [usd 0]}
              ] ""
-     assertBool "one zero posting is considered balanced for now" (isTransactionBalanced Nothing t)
-     let t = Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Unmarked "" "a" "" []
-             [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
-             ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
-             ,posting{paccount="d", pamount=Mixed [usd 100], ptype=VirtualPosting, ptransaction=Just t}
+     
+    ,test "virtual postings don't need to balance" $ expect $
+       isTransactionBalanced Nothing $ Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Unmarked "" "a" "" []
+             [posting{paccount="b", pamount=Mixed [usd 1.00]}
+             ,posting{paccount="c", pamount=Mixed [usd (-1.00)]}
+             ,posting{paccount="d", pamount=Mixed [usd 100], ptype=VirtualPosting}
              ] ""
-     assertBool "virtual postings don't need to balance" (isTransactionBalanced Nothing t)
-     let t = Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Unmarked "" "a" "" []
-             [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
-             ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
-             ,posting{paccount="d", pamount=Mixed [usd 100], ptype=BalancedVirtualPosting, ptransaction=Just t}
+     
+    ,test "balanced virtual postings need to balance among themselves" $ expect $
+       not $ isTransactionBalanced Nothing $ Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Unmarked "" "a" "" []
+             [posting{paccount="b", pamount=Mixed [usd 1.00]}
+             ,posting{paccount="c", pamount=Mixed [usd (-1.00)]}
+             ,posting{paccount="d", pamount=Mixed [usd 100], ptype=BalancedVirtualPosting}
              ] ""
-     assertBool "balanced virtual postings need to balance among themselves" (not $ isTransactionBalanced Nothing t)
-     let t = Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Unmarked "" "a" "" []
-             [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
-             ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
-             ,posting{paccount="d", pamount=Mixed [usd 100], ptype=BalancedVirtualPosting, ptransaction=Just t}
-             ,posting{paccount="3", pamount=Mixed [usd (-100)], ptype=BalancedVirtualPosting, ptransaction=Just t}
+     
+    ,test "balanced virtual postings need to balance among themselves (2)" $ expect $
+       isTransactionBalanced Nothing $ Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Unmarked "" "a" "" []
+             [posting{paccount="b", pamount=Mixed [usd 1.00]}
+             ,posting{paccount="c", pamount=Mixed [usd (-1.00)]}
+             ,posting{paccount="d", pamount=Mixed [usd 100], ptype=BalancedVirtualPosting}
+             ,posting{paccount="3", pamount=Mixed [usd (-100)], ptype=BalancedVirtualPosting}
              ] ""
-     assertBool "balanced virtual postings need to balance among themselves (2)" (isTransactionBalanced Nothing t)
+     
+  ]
 
-  ]]
+ ]
