@@ -1,4 +1,5 @@
-{-# LANGUAGE CPP, DeriveDataTypeable, StandaloneDeriving, DeriveGeneric, TypeSynonymInstances, FlexibleInstances, OverloadedStrings #-}
+{-# LANGUAGE DeriveDataTypeable, StandaloneDeriving, DeriveGeneric, TypeSynonymInstances, FlexibleInstances, OverloadedStrings #-}
+{-# LANGUAGE RecordWildCards #-}
 {-|
 
 Most data types are defined here to avoid import cycles.
@@ -25,17 +26,26 @@ import Control.DeepSeq (NFData)
 import Data.Data
 import Data.Decimal
 import Data.Default
+import Data.List (intercalate)
 import Text.Blaze (ToMarkup(..))
+--XXX https://hackage.haskell.org/package/containers/docs/Data-Map.html 
+--Note: You should use Data.Map.Strict instead of this module if:
+--You will eventually need all the values stored.
+--The stored values don't represent large virtual data structures to be lazily computed.
 import qualified Data.Map as M
 import Data.Text (Text)
 -- import qualified Data.Text as T
 import Data.Time.Calendar
 import Data.Time.LocalTime
 import System.Time (ClockTime(..))
+import Text.Printf
 
 import Hledger.Utils.Regex
 
 
+-- | A possibly incomplete date, whose missing parts will be filled from a reference date.
+-- A numeric year, month, and day of month, or the empty string for any of these.
+-- See the smartdate parser.
 type SmartDate = (String,String,String)
 
 data WhichDate = PrimaryDate | SecondaryDate deriving (Eq,Show)
@@ -103,7 +113,26 @@ instance Default Interval where def = NoInterval
 instance NFData Interval
 
 type AccountName = Text
-type AccountCode = Int
+
+data AccountType =
+    Asset
+  | Liability
+  | Equity
+  | Revenue
+  | Expense
+  deriving (Show,Eq,Ord,Data,Generic)
+
+instance NFData AccountType
+
+-- not worth the trouble, letters defined in accountdirectivep for now
+--instance Read AccountType
+--  where
+--    readsPrec _ ('A' : xs) = [(Asset,     xs)]
+--    readsPrec _ ('L' : xs) = [(Liability, xs)]
+--    readsPrec _ ('E' : xs) = [(Equity,    xs)]
+--    readsPrec _ ('R' : xs) = [(Revenue,   xs)]
+--    readsPrec _ ('X' : xs) = [(Expense,   xs)]
+--    readsPrec _ _ = []
 
 data AccountAlias = BasicAlias AccountName AccountName
                   | RegexAlias Regexp Replacement
@@ -125,8 +154,9 @@ instance ToMarkup Quantity
    toMarkup = toMarkup . show
 
 -- | An amount's price (none, per unit, or total) in another commodity.
--- Note the price should be a positive number, although this is not enforced.
-data Price = NoPrice | UnitPrice Amount | TotalPrice Amount deriving (Eq,Ord,Typeable,Data,Generic)
+-- The price amount should always be positive.
+data Price = NoPrice | UnitPrice Amount | TotalPrice Amount 
+  deriving (Eq,Ord,Typeable,Data,Generic,Show)
 
 instance NFData Price
 
@@ -137,10 +167,19 @@ data AmountStyle = AmountStyle {
       asprecision       :: !Int,                 -- ^ number of digits displayed after the decimal point
       asdecimalpoint    :: Maybe Char,           -- ^ character used as decimal point: period or comma. Nothing means "unspecified, use default"
       asdigitgroups     :: Maybe DigitGroupStyle -- ^ style for displaying digit groups, if any
-} deriving (Eq,Ord,Read,Show,Typeable,Data,Generic)
+} deriving (Eq,Ord,Read,Typeable,Data,Generic)
 
 instance NFData AmountStyle
 
+instance Show AmountStyle where
+  show AmountStyle{..} =
+    printf "AmountStylePP \"%s %s %s %s %s..\""
+    (show ascommodityside)
+    (show ascommodityspaced)
+    (show asprecision)
+    (show asdecimalpoint)
+    (show asdigitgroups)
+
 -- | A style for displaying digit groups in the integer part of a
 -- floating point number. It consists of the character used to
 -- separate groups (comma or period, whichever is not used as decimal
@@ -166,12 +205,13 @@ data Amount = Amount {
       aquantity   :: Quantity,
       aprice      :: Price,           -- ^ the (fixed) price for this amount, if any
       astyle      :: AmountStyle,
-      amultiplier :: Bool             -- ^ amount is a multipier for AutoTransactions
-    } deriving (Eq,Ord,Typeable,Data,Generic)
+      amultiplier :: Bool             -- ^ kludge: a flag marking this amount and posting as a multiplier
+                                      --   in a TMPostingRule. In a regular Posting, should always be false.
+    } deriving (Eq,Ord,Typeable,Data,Generic,Show)
 
 instance NFData Amount
 
-newtype MixedAmount = Mixed [Amount] deriving (Eq,Ord,Typeable,Data,Generic)
+newtype MixedAmount = Mixed [Amount] deriving (Eq,Ord,Typeable,Data,Generic,Show)
 
 instance NFData MixedAmount
 
@@ -197,7 +237,15 @@ instance Show Status where -- custom show.. bad idea.. don't do it..
   show Pending   = "!"
   show Cleared   = "*"
 
-type BalanceAssertion = Maybe (Amount, GenericSourcePos)
+-- | The amount to compare an account's balance to, to verify that the history
+-- leading to a given point is correct or to set the account to a known value.
+data BalanceAssertion = BalanceAssertion {
+      baamount   :: Amount,             -- ^ the expected value of a particular commodity
+      baexact    :: Bool,               -- ^ whether the assertion is exclusive, and doesn't allow other commodities alongside 'baamount'
+      baposition :: GenericSourcePos
+    } deriving (Eq,Typeable,Data,Generic,Show)
+
+instance NFData BalanceAssertion
 
 data Posting = Posting {
       pdate             :: Maybe Day,         -- ^ this posting's date, if different from the transaction's
@@ -207,20 +255,40 @@ data Posting = Posting {
       pamount           :: MixedAmount,
       pcomment          :: Text,              -- ^ this posting's comment lines, as a single non-indented multi-line string
       ptype             :: PostingType,
-      ptags             :: [Tag],             -- ^ tag names and values, extracted from the comment
-      pbalanceassertion :: BalanceAssertion,  -- ^ optional: the expected balance in this commodity in the account after this posting
-      ptransaction      :: Maybe Transaction, -- ^ this posting's parent transaction (co-recursive types).
-                                              -- Tying this knot gets tedious, Maybe makes it easier/optional.
-      porigin           :: Maybe Posting      -- ^ original posting if this one is result of any transformations (one level only)
+      ptags             :: [Tag],                   -- ^ tag names and values, extracted from the comment
+      pbalanceassertion :: Maybe BalanceAssertion,  -- ^ optional: the expected balance in this commodity in the account after this posting
+      ptransaction      :: Maybe Transaction,       -- ^ this posting's parent transaction (co-recursive types).
+                                                    --   Tying this knot gets tedious, Maybe makes it easier/optional.
+      porigin           :: Maybe Posting            -- ^ When this posting has been transformed in some way
+                                                    --   (eg its amount or price was inferred, or the account name was
+                                                    --   changed by a pivot or budget report), this references the original 
+                                                    --   untransformed posting (which will have Nothing in this field).
     } deriving (Typeable,Data,Generic)
 
 instance NFData Posting
 
 -- The equality test for postings ignores the parent transaction's
--- identity, to avoid infinite loops.
+-- identity, to avoid recuring ad infinitum.
+-- XXX could check that it's Just or Nothing.
 instance Eq Posting where
     (==) (Posting a1 b1 c1 d1 e1 f1 g1 h1 i1 _ _) (Posting a2 b2 c2 d2 e2 f2 g2 h2 i2 _ _) =  a1==a2 && b1==b2 && c1==c2 && d1==d2 && e1==e2 && f1==f2 && g1==g2 && h1==h2 && i1==i2
 
+-- | Posting's show instance elides the parent transaction so as not to recurse forever.
+instance Show Posting where
+  show Posting{..} = "PostingPP {" ++ intercalate ", " [
+     ("pdate="             ++ show (show pdate))
+    ,("pdate2="            ++ show (show pdate2))
+    ,("pstatus="           ++ show (show pstatus))
+    ,("paccount="          ++ show paccount)
+    ,("pamount="           ++ show pamount)
+    ,("pcomment="          ++ show pcomment)
+    ,("ptype="             ++ show ptype)
+    ,("ptags="             ++ show ptags)
+    ,("pbalanceassertion=" ++ show pbalanceassertion)
+    ,("ptransaction="      ++ show (const "<txn>" <$> ptransaction))
+    ,("porigin="           ++ show porigin)
+    ] ++ "}"
+
 -- TODO: needs renaming, or removal if no longer needed. See also TextPosition in Hledger.UI.Editor
 -- | The position of parse errors (eg), like parsec's SourcePos but generic.
 data GenericSourcePos = GenericSourcePos FilePath Int Int    -- ^ file path, 1-based line number and 1-based column number.
@@ -245,18 +313,34 @@ data Transaction = Transaction {
       ttags                    :: [Tag],     -- ^ tag names and values, extracted from the comment
       tpostings                :: [Posting], -- ^ this transaction's postings
       tpreceding_comment_lines :: Text       -- ^ any comment lines immediately preceding this transaction
-    } deriving (Eq,Typeable,Data,Generic)
+    } deriving (Eq,Typeable,Data,Generic,Show)
 
 instance NFData Transaction
 
-data ModifierTransaction = ModifierTransaction {
-      mtvalueexpr :: Text,
-      mtpostings  :: [Posting]
-    } deriving (Eq,Typeable,Data,Generic)
+-- | A transaction modifier rule. This has a query which matches postings
+-- in the journal, and a list of transformations to apply to those 
+-- postings or their transactions. Currently there is one kind of transformation:
+-- the TMPostingRule, which adds a posting ("auto posting") to the transaction, 
+-- optionally setting its amount to the matched posting's amount multiplied by a constant. 
+data TransactionModifier = TransactionModifier {
+      tmquerytxt :: Text,
+      tmpostingrules :: [TMPostingRule]
+    } deriving (Eq,Typeable,Data,Generic,Show)
 
-instance NFData ModifierTransaction
+instance NFData TransactionModifier
 
--- ^ A periodic transaction rule, describing a transaction that recurs.
+nulltransactionmodifier = TransactionModifier{
+  tmquerytxt = ""
+ ,tmpostingrules = []
+}
+
+-- | A transaction modifier transformation, which adds an extra posting
+-- to the matched posting's transaction.
+-- Can be like a regular posting, or the amount can have the amultiplier flag set,
+-- indicating that it's a multiplier for the matched posting's amount.
+type TMPostingRule = Posting
+
+-- | A periodic transaction rule, describing a transaction that recurs.
 data PeriodicTransaction = PeriodicTransaction {
       ptperiodexpr   :: Text,     -- ^ the period expression as written
       ptinterval     :: Interval, -- ^ the interval at which this transaction recurs 
@@ -268,7 +352,7 @@ data PeriodicTransaction = PeriodicTransaction {
       ptcomment      :: Text,
       pttags         :: [Tag],
       ptpostings     :: [Posting]
-    } deriving (Eq,Typeable,Data,Generic)
+    } deriving (Eq,Typeable,Data,Generic) -- , Show in PeriodicTransaction.hs
 
 nullperiodictransaction = PeriodicTransaction{
       ptperiodexpr   = ""
@@ -302,7 +386,7 @@ data MarketPrice = MarketPrice {
       mpdate      :: Day,
       mpcommodity :: CommoditySymbol,
       mpamount    :: Amount
-    } deriving (Eq,Ord,Typeable,Data,Generic) -- & Show (in Amount.hs)
+    } deriving (Eq,Ord,Typeable,Data,Generic) -- , Show in Amount.hs
 
 instance NFData MarketPrice
 
@@ -323,13 +407,15 @@ data Journal = Journal {
   ,jparseparentaccounts   :: [AccountName]                         -- ^ the current stack of parent account names, specified by apply account directives
   ,jparsealiases          :: [AccountAlias]                        -- ^ the current account name aliases in effect, specified by alias directives (& options ?)
   -- ,jparsetransactioncount :: Integer                               -- ^ the current count of transactions parsed so far (only journal format txns, currently)
-  ,jparsetimeclockentries :: [TimeclockEntry]                   -- ^ timeclock sessions which have not been clocked out
+  ,jparsetimeclockentries :: [TimeclockEntry]                       -- ^ timeclock sessions which have not been clocked out
+  ,jincludefilestack      :: [FilePath]
   -- principal data
-  ,jaccounts              :: [(AccountName, Maybe AccountCode)]     -- ^ accounts that have been declared by account directives
+  ,jdeclaredaccounts      :: [AccountName]                          -- ^ Accounts declared by account directives, in parse order (after journal finalisation) 
+  ,jdeclaredaccounttypes  :: M.Map AccountType [AccountName]        -- ^ Accounts whose type has been declared in account directives (usually 5 top-level accounts) 
   ,jcommodities           :: M.Map CommoditySymbol Commodity        -- ^ commodities and formats declared by commodity directives
-  ,jinferredcommodities   :: M.Map CommoditySymbol AmountStyle      -- ^ commodities and formats inferred from journal amounts  XXX misnamed
+  ,jinferredcommodities   :: M.Map CommoditySymbol AmountStyle      -- ^ commodities and formats inferred from journal amounts  TODO misnamed - jusedstyles
   ,jmarketprices          :: [MarketPrice]
-  ,jmodifiertxns          :: [ModifierTransaction]
+  ,jtxnmodifiers          :: [TransactionModifier]
   ,jperiodictxns          :: [PeriodicTransaction]
   ,jtxns                  :: [Transaction]
   ,jfinalcommentlines     :: Text                                   -- ^ any final trailing comments in the (main) journal file
@@ -357,7 +443,7 @@ type StorageFormat = String
 -- which let you walk up or down the account tree.
 data Account = Account {
   aname                     :: AccountName,   -- ^ this account's full name
-  acode                     :: Maybe AccountCode,   -- ^ this account's numeric code, if any (not always set) 
+  adeclarationorder         :: Maybe Int  ,   -- ^ the relative position of this account's account directive, if any. Normally a natural number. 
   aebalance                 :: MixedAmount,   -- ^ this account's balance, excluding subaccounts
   asubs                     :: [Account],     -- ^ sub-accounts
   anumpostings              :: Int,           -- ^ number of postings to this account
