@@ -29,8 +29,7 @@ module Hledger.Read (
   module Hledger.Read.Common,
 
   -- * Tests
-  samplejournal,
-  tests_Hledger_Read,
+  tests_Read,
 
 ) where
 
@@ -50,17 +49,16 @@ import System.Environment (getEnv)
 import System.Exit (exitFailure)
 import System.FilePath
 import System.IO
-import Test.HUnit
 import Text.Printf
 
 import Hledger.Data.Dates (getCurrentDay, parsedate, showDate)
 import Hledger.Data.Types
 import Hledger.Read.Common
-import qualified Hledger.Read.JournalReader   as JournalReader
+import Hledger.Read.JournalReader   as JournalReader
 -- import qualified Hledger.Read.LedgerReader    as LedgerReader
 import qualified Hledger.Read.TimedotReader   as TimedotReader
 import qualified Hledger.Read.TimeclockReader as TimeclockReader
-import qualified Hledger.Read.CsvReader       as CsvReader
+import Hledger.Read.CsvReader as CsvReader
 import Hledger.Utils
 import Prelude hiding (getContents, writeFile)
 
@@ -149,12 +147,6 @@ newJournalContent = do
 readJournal' :: Text -> IO Journal
 readJournal' t = readJournal def Nothing t >>= either error' return
 
-tests_readJournal' = [
-  "readJournal' parses sample journal" ~: do
-     _ <- samplejournal
-     assertBool "" True
- ]
-
 -- | @findReader mformat mpath@
 --
 -- Find the reader named by @mformat@, if provided.
@@ -315,47 +307,37 @@ tryReaders iopts mpath readers txt = firstSuccessOrFirstError [] readers
 
 -- tests
 
-samplejournal = readJournal' $ T.unlines
- ["2008/01/01 income"
- ,"    assets:bank:checking  $1"
- ,"    income:salary"
- ,""
- ,"comment"
- ,"multi line comment here"
- ,"for testing purposes"
- ,"end comment"
- ,""
- ,"2008/06/01 gift"
- ,"    assets:bank:checking  $1"
- ,"    income:gifts"
- ,""
- ,"2008/06/02 save"
- ,"    assets:bank:saving  $1"
- ,"    assets:bank:checking"
- ,""
- ,"2008/06/03 * eat & shop"
- ,"    expenses:food      $1"
- ,"    expenses:supplies  $1"
- ,"    assets:cash"
- ,""
- ,"2008/12/31 * pay off"
- ,"    liabilities:debts  $1"
- ,"    assets:bank:checking"
- ]
-
-tests_Hledger_Read = TestList $
-  tests_readJournal'
-  ++ [
-   JournalReader.tests_Hledger_Read_JournalReader,
---    LedgerReader.tests_Hledger_Read_LedgerReader,
-   TimeclockReader.tests_Hledger_Read_TimeclockReader,
-   TimedotReader.tests_Hledger_Read_TimedotReader,
-   CsvReader.tests_Hledger_Read_CsvReader,
-
-   "journal" ~: do
-    r <- runExceptT $ parseWithState mempty JournalReader.journalp ""
-    assertBool "journalp should parse an empty file" (isRight $ r)
-    jE <- readJournal def Nothing "" -- don't know how to get it from journal
-    either error' (assertBool "journalp parsing an empty file should give an empty journal" . null . jtxns) jE
-
+tests_Read = tests "Read" [
+   tests_Common
+  ,tests_CsvReader
+  ,tests_JournalReader
   ]
+
+--samplejournal = readJournal' $ T.unlines
+-- ["2008/01/01 income"
+-- ,"    assets:bank:checking  $1"
+-- ,"    income:salary"
+-- ,""
+-- ,"comment"
+-- ,"multi line comment here"
+-- ,"for testing purposes"
+-- ,"end comment"
+-- ,""
+-- ,"2008/06/01 gift"
+-- ,"    assets:bank:checking  $1"
+-- ,"    income:gifts"
+-- ,""
+-- ,"2008/06/02 save"
+-- ,"    assets:bank:saving  $1"
+-- ,"    assets:bank:checking"
+-- ,""
+-- ,"2008/06/03 * eat & shop"
+-- ,"    expenses:food      $1"
+-- ,"    expenses:supplies  $1"
+-- ,"    assets:cash"
+-- ,""
+-- ,"2008/12/31 * pay off"
+-- ,"    liabilities:debts  $1"
+-- ,"    assets:bank:checking"
+-- ]
+
