@@ -29,18 +29,21 @@ module Hledger.Read.Common (
   rtp,
   runJournalParser,
   rjp,
+  runErroringJournalParser,
+  rejp,
   genericSourcePos,
   journalSourcePos,
-  generateAutomaticPostings,
+  applyTransactionModifiers,
   parseAndFinaliseJournal,
-  parseAndFinaliseJournal',  -- TODO unused ? check addons
+  parseAndFinaliseJournal',
   setYear,
   getYear,
   setDefaultCommodityAndStyle,
   getDefaultCommodityAndStyle,
   getDefaultAmountStyle,
   getAmountStyle,
-  pushAccount,
+  pushDeclaredAccount,
+  addDeclaredAccountType,
   pushParentAccount,
   popParentAccount,
   getParentAccount,
@@ -71,7 +74,7 @@ module Hledger.Read.Common (
   mamountp',
   commoditysymbolp,
   priceamountp,
-  partialbalanceassertionp,
+  balanceassertionp,
   fixedlotpricep,
   numberp,
   fromRawNumber,
@@ -90,14 +93,18 @@ module Hledger.Read.Common (
 
   -- ** misc
   singlespacedtextp,
-  singlespacep
+  singlespacedtextsatisfyingp,
+  singlespacep,
+
+  -- * tests
+  tests_Common,
 )
 where
 --- * imports
 import Prelude ()
 import "base-compat-batteries" Prelude.Compat hiding (readFile)
 import "base-compat-batteries" Control.Monad.Compat
-import Control.Monad.Except (ExceptT(..), throwError)
+import Control.Monad.Except (ExceptT(..), runExceptT, throwError)
 import Control.Monad.State.Strict
 import Data.Bifunctor (bimap, second)
 import Data.Char
@@ -122,7 +129,9 @@ import Text.Megaparsec.Custom
 
 import Hledger.Data
 import Hledger.Utils
-import qualified Hledger.Query as Q (Query(Any))
+
+-- $setup
+-- >>> :set -XOverloadedStrings
 
 -- | A hledger journal reader is a triple of storage format name, a
 -- detector of that format, and a parser from that format to Journal.
@@ -154,6 +163,7 @@ data InputOpts = InputOpts {
      mformat_           :: Maybe StorageFormat  -- ^ a file/storage format to try, unless overridden
                                                 --   by a filename prefix. Nothing means try all.
     ,mrules_file_       :: Maybe FilePath       -- ^ a conversion rules file to use (when reading CSV)
+    ,separator_         :: Char                 -- ^ the separator to use (when reading CSV)
     ,aliases_           :: [String]             -- ^ account name aliases to apply
     ,anon_              :: Bool                 -- ^ do light anonymisation/obfuscation of the data 
     ,ignore_assertions_ :: Bool                 -- ^ don't check balance assertions
@@ -166,13 +176,14 @@ data InputOpts = InputOpts {
 instance Default InputOpts where def = definputopts
 
 definputopts :: InputOpts
-definputopts = InputOpts def def def def def def True def def
+definputopts = InputOpts def def ',' def def def def True def def
 
 rawOptsToInputOpts :: RawOpts -> InputOpts
 rawOptsToInputOpts rawopts = InputOpts{
    -- files_             = map (T.unpack . stripquotes . T.pack) $ listofstringopt "file" rawopts
    mformat_           = Nothing
   ,mrules_file_       = maybestringopt "rules-file" rawopts
+  ,separator_         = fromMaybe ',' (maybecharopt "separator" rawopts)
   ,aliases_           = map (T.unpack . stripquotes . T.pack) $ listofstringopt "alias" rawopts
   ,anon_              = boolopt "anon" rawopts
   ,ignore_assertions_ = boolopt "ignore-assertions" rawopts
@@ -184,16 +195,29 @@ rawOptsToInputOpts rawopts = InputOpts{
 
 --- * parsing utilities
 
--- | Run a string parser with no state in the identity monad.
-runTextParser, rtp :: TextParser Identity a -> Text -> Either (ParseError Char CustomErr) a
+-- | Run a text parser in the identity monad. See also: parseWithState.
+runTextParser, rtp
+  :: TextParser Identity a -> Text -> Either (ParseErrorBundle Text CustomErr) a
 runTextParser p t =  runParser p "" t
 rtp = runTextParser
 
--- | Run a journal parser with a null journal-parsing state.
-runJournalParser, rjp :: Monad m => JournalParser m a -> Text -> m (Either (ParseError Char CustomErr) a)
+-- | Run a journal parser in some monad. See also: parseWithState.
+runJournalParser, rjp
+  :: Monad m
+  => JournalParser m a -> Text -> m (Either (ParseErrorBundle Text CustomErr) a)
 runJournalParser p t = runParserT (evalStateT p mempty) "" t
 rjp = runJournalParser
 
+-- | Run an erroring journal parser in some monad. See also: parseWithState.
+runErroringJournalParser, rejp
+  :: Monad m
+  => ErroringJournalParser m a
+  -> Text
+  -> m (Either FinalParseError (Either (ParseErrorBundle Text CustomErr) a))
+runErroringJournalParser p t =
+  runExceptT $ runParserT (evalStateT p mempty) "" t
+rejp = runErroringJournalParser
+
 genericSourcePos :: SourcePos -> GenericSourcePos
 genericSourcePos p = GenericSourcePos (sourceName p) (fromIntegral . unPos $ sourceLine p) (fromIntegral . unPos $ sourceColumn p)
 
@@ -205,43 +229,98 @@ journalSourcePos p p' = JournalSourcePos (sourceName p) (fromIntegral . unPos $
             | otherwise = unPos $ sourceLine p' -- might be at end of file withat last new-line
 
 
--- | Generate Automatic postings and add them to the current journal.
-generateAutomaticPostings :: Journal -> Journal
-generateAutomaticPostings j = j { jtxns = map modifier $ jtxns j }
+-- | Apply any transaction modifier rules in the journal 
+-- (adding automated postings to transactions, eg).
+applyTransactionModifiers :: Journal -> Journal
+applyTransactionModifiers j = j { jtxns = map applyallmodifiers $ jtxns j }
   where
-    modifier = foldr (flip (.) . runModifierTransaction') id mtxns
-    runModifierTransaction' = fmap txnTieKnot . runModifierTransaction Q.Any
-    mtxns = jmodifiertxns j
+    applyallmodifiers = 
+      foldr (flip (.) . transactionModifierToFunction) id (jtxnmodifiers j)
 
 -- | Given a megaparsec ParsedJournal parser, input options, file
 -- path and file content: parse and post-process a Journal, or give an error.
-parseAndFinaliseJournal :: JournalParser IO ParsedJournal -> InputOpts
+parseAndFinaliseJournal :: ErroringJournalParser IO ParsedJournal -> InputOpts
                            -> FilePath -> Text -> ExceptT String IO Journal
 parseAndFinaliseJournal parser iopts f txt = do
   t <- liftIO getClockTime
   y <- liftIO getCurrentYear
-  ep <- liftIO $ runParserT (evalStateT parser nulljournal {jparsedefaultyear=Just y}) f txt
-  case ep of
-    Right pj -> 
-      let pj' = if auto_ iopts then generateAutomaticPostings pj else pj in
-      case journalFinalise t f txt (not $ ignore_assertions_ iopts) pj' of
-                        Right j -> return j
-                        Left e  -> throwError e
-    Left e   -> throwError $ customParseErrorPretty txt e
+  let initJournal = nulljournal
+        { jparsedefaultyear = Just y
+        , jincludefilestack = [f] }
+  eep <- liftIO $ runExceptT $
+    runParserT (evalStateT parser initJournal) f txt
+  case eep of
+    Left finalParseError ->
+      throwError $ finalErrorBundlePretty $ attachSource f txt finalParseError
 
-parseAndFinaliseJournal' :: JournalParser Identity ParsedJournal -> InputOpts 
-                            -> FilePath -> Text -> ExceptT String IO Journal
+    Right ep -> case ep of
+      Left e -> throwError $ customErrorBundlePretty e
+
+      Right pj ->
+        -- If we are using automated transactions, we finalize twice:
+        -- once before and once after. However, if we are running it
+        -- twice, we don't check assertions the first time (they might
+        -- be false pending modifiers) and we don't reorder the second
+        -- time. If we are only running once, we reorder and follow
+        -- the options for checking assertions.
+        let fj = if auto_ iopts && (not . null . jtxnmodifiers) pj
+                 then applyTransactionModifiers <$>
+                      (journalBalanceTransactions False $
+                       journalReverse $
+                       journalApplyCommodityStyles pj) >>=
+                      (\j -> journalBalanceTransactions (not $ ignore_assertions_ iopts) $
+                             journalAddFile (f, txt) $
+                             journalSetLastReadTime t $
+                             j)
+                 else journalBalanceTransactions (not $ ignore_assertions_ iopts) $
+                      journalReverse $
+                      journalAddFile (f, txt) $
+                      journalApplyCommodityStyles $
+                      journalSetLastReadTime t $
+                      pj
+        in
+          case fj of
+            Right j -> return j
+            Left e  -> throwError e
+
+parseAndFinaliseJournal' :: JournalParser IO ParsedJournal -> InputOpts
+                           -> FilePath -> Text -> ExceptT String IO Journal
 parseAndFinaliseJournal' parser iopts f txt = do
   t <- liftIO getClockTime
   y <- liftIO getCurrentYear
-  let ep = runParser (evalStateT parser nulljournal {jparsedefaultyear=Just y}) f txt
+  let initJournal = nulljournal
+        { jparsedefaultyear = Just y
+        , jincludefilestack = [f] }
+  ep <- liftIO $ runParserT (evalStateT parser initJournal) f txt
   case ep of
-    Right pj -> 
-      let pj' = if auto_ iopts then generateAutomaticPostings pj else pj in      
-      case journalFinalise t f txt (not $ ignore_assertions_ iopts) pj' of
-                        Right j -> return j
-                        Left e  -> throwError e
-    Left e   -> throwError $ parseErrorPretty e
+    Left e   -> throwError $ customErrorBundlePretty e
+
+    Right pj ->
+      -- If we are using automated transactions, we finalize twice:
+      -- once before and once after. However, if we are running it
+      -- twice, we don't check assertions the first time (they might
+      -- be false pending modifiers) and we don't reorder the second
+      -- time. If we are only running once, we reorder and follow the
+      -- options for checking assertions.
+      let fj = if auto_ iopts && (not . null . jtxnmodifiers) pj
+               then applyTransactionModifiers <$>
+                    (journalBalanceTransactions False $
+                     journalReverse $
+                     journalApplyCommodityStyles pj) >>=
+                    (\j -> journalBalanceTransactions (not $ ignore_assertions_ iopts) $
+                           journalAddFile (f, txt) $
+                           journalSetLastReadTime t $
+                           j)
+               else journalBalanceTransactions (not $ ignore_assertions_ iopts) $
+                    journalReverse $
+                    journalAddFile (f, txt) $
+                    journalApplyCommodityStyles $
+                    journalSetLastReadTime t $
+                    pj
+      in
+        case fj of
+          Right j -> return j
+          Left e  -> throwError e
 
 setYear :: Year -> JournalParser m ()
 setYear y = modify' (\j -> j{jparsedefaultyear=Just y})
@@ -273,8 +352,12 @@ getAmountStyle commodity = do
     let effectiveStyle = listToMaybe $ catMaybes [specificStyle, defaultStyle]
     return effectiveStyle
 
-pushAccount :: AccountName -> JournalParser m ()
-pushAccount acct = modify' (\j -> j{jaccounts = (acct, Nothing) : jaccounts j})
+pushDeclaredAccount :: AccountName -> JournalParser m ()
+pushDeclaredAccount acct = modify' (\j -> j{jdeclaredaccounts = acct : jdeclaredaccounts j})
+
+addDeclaredAccountType :: AccountName -> AccountType -> JournalParser m ()
+addDeclaredAccountType acct atype = 
+  modify' (\j -> j{jdeclaredaccounttypes = M.insertWith (++) atype [acct] (jdeclaredaccounttypes j)})
 
 pushParentAccount :: AccountName -> JournalParser m ()
 pushParentAccount acct = modify' (\j -> j{jparseparentaccounts = acct : jparseparentaccounts j})
@@ -353,43 +436,43 @@ datep = do
 
 datep' :: Maybe Year -> TextParser m Day
 datep' mYear = do
-  startPos <- getPosition
+  startOffset <- getOffset
   d1 <- decimal <?> "year or month"
   sep <- satisfy isDateSepChar <?> "date separator"
   d2 <- decimal <?> "month or day"
-  fullDate startPos d1 sep d2 <|> partialDate startPos mYear d1 sep d2
+  fullDate startOffset d1 sep d2 <|> partialDate startOffset mYear d1 sep d2
   <?> "full or partial date"
 
   where
 
-  fullDate :: SourcePos -> Integer -> Char -> Int -> TextParser m Day
-  fullDate startPos year sep1 month = do
+  fullDate :: Int -> Integer -> Char -> Int -> TextParser m Day
+  fullDate startOffset year sep1 month = do
     sep2 <- satisfy isDateSepChar <?> "date separator"
     day <- decimal <?> "day"
-    endPos <- getPosition
+    endOffset <- getOffset
     let dateStr = show year ++ [sep1] ++ show month ++ [sep2] ++ show day
 
-    when (sep1 /= sep2) $ parseErrorAtRegion startPos endPos $
+    when (sep1 /= sep2) $ customFailure $ parseErrorAtRegion startOffset endOffset $
       "invalid date (mixing date separators is not allowed): " ++ dateStr
 
     case fromGregorianValid year month day of
-      Nothing -> parseErrorAtRegion startPos endPos $
+      Nothing -> customFailure $ parseErrorAtRegion startOffset endOffset $
                    "well-formed but invalid date: " ++ dateStr
       Just date -> pure $! date
 
   partialDate
-    :: SourcePos -> Maybe Year -> Integer -> Char -> Int -> TextParser m Day
-  partialDate startPos mYear month sep day = do
-    endPos <- getPosition
+    :: Int -> Maybe Year -> Integer -> Char -> Int -> TextParser m Day
+  partialDate startOffset mYear month sep day = do
+    endOffset <- getOffset
     case mYear of
       Just year ->
         case fromGregorianValid year (fromIntegral month) day of
-          Nothing -> parseErrorAtRegion startPos endPos $
+          Nothing -> customFailure $ parseErrorAtRegion startOffset endOffset $
                       "well-formed but invalid date: " ++ dateStr
           Just date -> pure $! date
         where dateStr = show year ++ [sep] ++ show month ++ [sep] ++ show day
 
-      Nothing -> parseErrorAtRegion startPos endPos $
+      Nothing -> customFailure $ parseErrorAtRegion startOffset endOffset $
         "partial date "++dateStr++" found, but the current year is unknown"
         where dateStr = show month ++ [sep] ++ show day
 
@@ -417,26 +500,27 @@ datetimep' mYear = do
   where
     timeOfDay :: TextParser m TimeOfDay
     timeOfDay = do
-      pos1 <- getPosition
+      off1 <- getOffset
       h' <- twoDigitDecimal <?> "hour"
-      pos2 <- getPosition
-      unless (h' >= 0 && h' <= 23) $ parseErrorAtRegion pos1 pos2
-        "invalid time (bad hour)"
+      off2 <- getOffset
+      unless (h' >= 0 && h' <= 23) $ customFailure $
+        parseErrorAtRegion off1 off2 "invalid time (bad hour)"
 
       char ':' <?> "':' (hour-minute separator)"
-      pos3 <- getPosition
+      off3 <- getOffset
       m' <- twoDigitDecimal <?> "minute"
-      pos4 <- getPosition
-      unless (m' >= 0 && m' <= 59) $ parseErrorAtRegion pos3 pos4
-        "invalid time (bad minute)"
+      off4 <- getOffset
+      unless (m' >= 0 && m' <= 59) $ customFailure $
+        parseErrorAtRegion off3 off4 "invalid time (bad minute)"
 
       s' <- option 0 $ do
         char ':' <?> "':' (minute-second separator)"
-        pos5 <- getPosition
+        off5 <- getOffset
         s' <- twoDigitDecimal <?> "second"
-        pos6 <- getPosition
-        unless (s' >= 0 && s' <= 59) $ parseErrorAtRegion pos5 pos6
-          "invalid time (bad second)" -- we do not support leap seconds
+        off6 <- getOffset
+        unless (s' >= 0 && s' <= 59) $ customFailure $
+          parseErrorAtRegion off5 off6 "invalid time (bad second)"
+          -- we do not support leap seconds
         pure s'
 
       pure $ TimeOfDay h' m' (fromIntegral s')
@@ -474,24 +558,35 @@ modifiedaccountnamep = do
     a
 
 -- | Parse an account name, plus one following space if present. 
--- Account names start with a non-space, may have single spaces inside them, 
+-- Account names have one or more parts separated by the account separator character,
 -- and are terminated by two or more spaces (or end of input). 
--- (Also they have one or more components of at least one character, 
--- separated by the account separator character, but we don't check that here.) 
+-- Each part is at least one character long, may have single spaces inside it,
+-- and starts with a non-whitespace.
+-- Note, this means "{account}", "%^!" and ";comment" are all accepted
+-- (parent parsers usually prevent/consume the last).
+-- It should have required parts to start with an alphanumeric;
+-- for now it remains as-is for backwards compatibility.
 accountnamep :: TextParser m AccountName
 accountnamep = singlespacedtextp
 
--- | Parse any text beginning with a non-whitespace character, until a double space or the end of input.
--- Consumes one of the following spaces, if present.
+
+-- | Parse any text beginning with a non-whitespace character, until a
+-- double space or the end of input.
 singlespacedtextp :: TextParser m T.Text
-singlespacedtextp = do
-  firstPart <- part
-  otherParts <- many $ try $ singlespacep *> part
+singlespacedtextp = singlespacedtextsatisfyingp (const True)
+
+-- | Similar to 'singlespacedtextp', except that the text must only contain
+-- characters satisfying the given predicate.
+singlespacedtextsatisfyingp :: (Char -> Bool) -> TextParser m T.Text
+singlespacedtextsatisfyingp pred = do
+  firstPart <- partp
+  otherParts <- many $ try $ singlespacep *> partp
   pure $! T.unwords $ firstPart : otherParts
   where
-    part = takeWhile1P Nothing (not . isSpace)
+    partp = takeWhile1P Nothing (\c -> pred c && not (isSpace c))
 
 -- | Parse one non-newline whitespace character that is not followed by another one.
+singlespacep :: TextParser m ()
 singlespacep = void spacenonewline *> notFollowedBy spacenonewline
 
 --- ** amounts
@@ -505,20 +600,6 @@ spaceandamountormissingp =
     lift $ skipSome spacenonewline
     Mixed . (:[]) <$> amountp
 
-#ifdef TESTS
-assertParseEqual' :: (Show a, Eq a) => (Either ParseError a) -> a -> Assertion
-assertParseEqual' parse expected = either (assertFailure.show) (`is'` expected) parse
-
-is' :: (Eq a, Show a) => a -> a -> Assertion
-a `is'` e = assertEqual e a
-
-test_spaceandamountormissingp = do
-    assertParseEqual' (parseWithState mempty spaceandamountormissingp " $47.18") (Mixed [usd 47.18])
-    assertParseEqual' (parseWithState mempty spaceandamountormissingp "$47.18") missingmixedamt
-    assertParseEqual' (parseWithState mempty spaceandamountormissingp " ") missingmixedamt
-    assertParseEqual' (parseWithState mempty spaceandamountormissingp "") missingmixedamt
-#endif
-
 -- | Parse a single-commodity amount, with optional symbol on the left or
 -- right, optional unit or total price, and optional (ignored)
 -- ledger-style balance assertion or fixed lot price declaration.
@@ -540,58 +621,48 @@ amountwithoutpricep = do
   leftsymbolamountp mult sign = label "amount" $ do
     c <- lift commoditysymbolp
     suggestedStyle <- getAmountStyle c
-
     commodityspaced <- lift $ skipMany' spacenonewline
-
     sign2 <- lift $ signp
-    posBeforeNum <- getPosition
+    offBeforeNum <- getOffset
     ambiguousRawNum <- lift rawnumberp
     mExponent <- lift $ optional $ try exponentp
-    posAfterNum <- getPosition
-    let numRegion = (posBeforeNum, posAfterNum)
-
-    (q,prec,mdec,mgrps) <- lift $
-      interpretNumber numRegion suggestedStyle ambiguousRawNum mExponent
+    offAfterNum <- getOffset
+    let numRegion = (offBeforeNum, offAfterNum)
+    (q,prec,mdec,mgrps) <- lift $ interpretNumber numRegion suggestedStyle ambiguousRawNum mExponent
     let s = amountstyle{ascommodityside=L, ascommodityspaced=commodityspaced, asprecision=prec, asdecimalpoint=mdec, asdigitgroups=mgrps}
     return $ Amount c (sign (sign2 q)) NoPrice s mult
 
-  rightornosymbolamountp
-    :: Bool -> (Decimal -> Decimal) -> JournalParser m Amount
+  rightornosymbolamountp :: Bool -> (Decimal -> Decimal) -> JournalParser m Amount
   rightornosymbolamountp mult sign = label "amount" $ do
-    posBeforeNum <- getPosition
+    offBeforeNum <- getOffset
     ambiguousRawNum <- lift rawnumberp
     mExponent <- lift $ optional $ try exponentp
-    posAfterNum <- getPosition
-    let numRegion = (posBeforeNum, posAfterNum)
-
-    mSpaceAndCommodity <- lift $ optional $ try $
-      (,) <$> skipMany' spacenonewline <*> commoditysymbolp
-
+    offAfterNum <- getOffset
+    let numRegion = (offBeforeNum, offAfterNum)
+    mSpaceAndCommodity <- lift $ optional $ try $ (,) <$> skipMany' spacenonewline <*> commoditysymbolp
     case mSpaceAndCommodity of
+      -- right symbol amount
       Just (commodityspaced, c) -> do
         suggestedStyle <- getAmountStyle c
-        (q,prec,mdec,mgrps) <- lift $
-          interpretNumber numRegion suggestedStyle ambiguousRawNum mExponent
-
+        (q,prec,mdec,mgrps) <- lift $ interpretNumber numRegion suggestedStyle ambiguousRawNum mExponent
         let s = amountstyle{ascommodityside=R, ascommodityspaced=commodityspaced, asprecision=prec, asdecimalpoint=mdec, asdigitgroups=mgrps}
         return $ Amount c (sign q) NoPrice s mult
-
+      -- no symbol amount
       Nothing -> do
         suggestedStyle <- getDefaultAmountStyle
-        (q,prec,mdec,mgrps) <- lift $
-          interpretNumber numRegion suggestedStyle ambiguousRawNum mExponent
-
-        -- apply the most recently seen default commodity and style to this commodityless amount
+        (q,prec,mdec,mgrps) <- lift $ interpretNumber numRegion suggestedStyle ambiguousRawNum mExponent
+        -- if a default commodity has been set, apply it and its style to this amount
+        -- (unless it's a multiplier in an automated posting)
         defcs <- getDefaultCommodityAndStyle
-        let (c,s) = case defcs of
-              Just (defc,defs) -> (defc, defs{asprecision=max (asprecision defs) prec})
-              Nothing          -> ("", amountstyle{asprecision=prec, asdecimalpoint=mdec, asdigitgroups=mgrps})
+        let (c,s) = case (mult, defcs) of
+              (False, Just (defc,defs)) -> (defc, defs{asprecision=max (asprecision defs) prec})
+              _ -> ("", amountstyle{asprecision=prec, asdecimalpoint=mdec, asdigitgroups=mgrps})
         return $ Amount c (sign q) NoPrice s mult
 
   -- For reducing code duplication. Doesn't parse anything. Has the type
   -- of a parser only in order to throw parse errors (for convenience).
   interpretNumber
-    :: (SourcePos, SourcePos)
+    :: (Int, Int) -- offsets
     -> Maybe AmountStyle
     -> Either AmbiguousNumber RawNumber
     -> Maybe Int
@@ -599,24 +670,10 @@ amountwithoutpricep = do
   interpretNumber posRegion suggestedStyle ambiguousNum mExp =
     let rawNum = either (disambiguateNumber suggestedStyle) id ambiguousNum
     in  case fromRawNumber rawNum mExp of
-          Left errMsg -> uncurry parseErrorAtRegion posRegion errMsg
+          Left errMsg -> customFailure $
+                           uncurry parseErrorAtRegion posRegion errMsg
           Right res -> pure res
 
-
-#ifdef TESTS
-test_amountp = do
-    assertParseEqual' (parseWithState mempty amountp "$47.18") (usd 47.18)
-    assertParseEqual' (parseWithState mempty amountp "$1.") (usd 1 `withPrecision` 0)
-  -- ,"amount with unit price" ~: do
-    assertParseEqual'
-     (parseWithState mempty amountp "$10 @ €0.5")
-     (usd 10 `withPrecision` 0 `at` (eur 0.5 `withPrecision` 1))
-  -- ,"amount with total price" ~: do
-    assertParseEqual'
-     (parseWithState mempty amountp "$10 @@ €5")
-     (usd 10 `withPrecision` 0 @@ (eur 5 `withPrecision` 0))
-#endif
-
 -- | Parse an amount from a string, or get an error.
 amountp' :: String -> Amount
 amountp' s =
@@ -668,26 +725,18 @@ priceamountp = option NoPrice $ do
 
   pure $ priceConstructor priceAmount
 
-partialbalanceassertionp :: JournalParser m BalanceAssertion
-partialbalanceassertionp = optional $ do
-  sourcepos <- try $ do
-    lift (skipMany spacenonewline)
-    sourcepos <- genericSourcePos <$> lift getPosition
-    char '='
-    pure sourcepos
+balanceassertionp :: JournalParser m BalanceAssertion
+balanceassertionp = do
+  sourcepos <- genericSourcePos <$> lift getSourcePos
+  char '='
+  exact <- optional $ try $ char '='
   lift (skipMany spacenonewline)
   a <- amountp <?> "amount (for a balance assertion or assignment)" -- XXX should restrict to a simple amount
-  return (a, sourcepos)
-
--- balanceassertion :: Monad m => TextParser m (Maybe MixedAmount)
--- balanceassertion =
---     try (do
---           lift (skipMany spacenonewline)
---           string "=="
---           lift (skipMany spacenonewline)
---           a <- amountp -- XXX should restrict to a simple amount
---           return $ Just $ Mixed [a])
---          <|> return Nothing
+  return BalanceAssertion
+    { baamount = a
+    , baexact = isJust exact
+    , baposition = sourcepos
+    }
 
 -- http://ledger-cli.org/3.0/doc/ledger3.html#Fixing-Lot-Prices
 fixedlotpricep :: JournalParser m (Maybe Amount)
@@ -719,7 +768,7 @@ numberp :: Maybe AmountStyle -> TextParser m (Quantity, Int, Maybe Char, Maybe D
 numberp suggestedStyle = label "number" $ do
     -- a number is an optional sign followed by a sequence of digits possibly
     -- interspersed with periods, commas, or both
-    -- ptrace "numberp"
+    -- dbgparse 0 "numberp"
     sign <- signp
     rawNum <- either (disambiguateNumber suggestedStyle) id <$> rawnumberp
     mExp <- optional $ try $ exponentp
@@ -831,9 +880,10 @@ rawnumberp = label "number" $ do
     fail "invalid number (invalid use of separator)"
 
   mExtraFragment <- optional $ lookAhead $ try $
-    char ' ' *> getPosition <* digitChar
+    char ' ' *> getOffset <* digitChar
   case mExtraFragment of
-    Just pos -> parseErrorAt pos "invalid number (excessive trailing digits)"
+    Just off -> customFailure $
+                  parseErrorAt off "invalid number (excessive trailing digits)"
     Nothing -> pure ()
 
   return $ dbg8 "rawnumberp" rawNumber
@@ -917,7 +967,6 @@ digitgroupp = label "digits"
     makeGroup = uncurry DigitGrp . foldl' step (0, 0) . T.unpack
     step (!l, !a) c = (l+1, a*10 + fromIntegral (digitToInt c))
 
-
 data RawNumber
   = NoSeparators   DigitGrp (Maybe (Char, DigitGrp))        -- 100 or 100. or .100 or 100.50
   | WithSeparators Char [DigitGrp] (Maybe (Char, DigitGrp)) -- 1,000,000 or 1,000.50
@@ -926,28 +975,6 @@ data RawNumber
 data AmbiguousNumber = AmbiguousNumber DigitGrp Char DigitGrp  -- 1,000
   deriving (Show, Eq)
 
--- test_numberp = do
---       let s `is` n = assertParseEqual (parseWithState mempty numberp s) n
---           assertFails = assertBool . isLeft . parseWithState mempty numberp
---       assertFails ""
---       "0"          `is` (0, 0, '.', ',', [])
---       "1"          `is` (1, 0, '.', ',', [])
---       "1.1"        `is` (1.1, 1, '.', ',', [])
---       "1,000.1"    `is` (1000.1, 1, '.', ',', [3])
---       "1.00.000,1" `is` (100000.1, 1, ',', '.', [3,2])
---       "1,000,000"  `is` (1000000, 0, '.', ',', [3,3])
---       "1."         `is` (1,   0, '.', ',', [])
---       "1,"         `is` (1,   0, ',', '.', [])
---       ".1"         `is` (0.1, 1, '.', ',', [])
---       ",1"         `is` (0.1, 1, ',', '.', [])
---       assertFails "1,000.000,1"
---       assertFails "1.000,000.1"
---       assertFails "1,000.000.1"
---       assertFails "1,,1"
---       assertFails "1..1"
---       assertFails ".1,"
---       assertFails ",1."
-
 --- ** comments
 
 multilinecommentp :: TextParser m ()
@@ -989,26 +1016,46 @@ emptyorcommentlinep = do
 -- until the next newline. This parser should extract the "content" from
 -- comments. The resulting parser returns this content plus the raw text
 -- of the comment itself.
-followingcommentp' :: (Monoid a) => TextParser m a -> TextParser m (Text, a)
+--
+-- See followingcommentp for tests.
+--
+followingcommentp' :: (Monoid a, Show a) => TextParser m a -> TextParser m (Text, a)
 followingcommentp' contentp = do
   skipMany spacenonewline
-  sameLine <- try headerp *> match' contentp <|> pure ("", mempty)
+  -- there can be 0 or 1 sameLine
+  sameLine <- try headerp *> ((:[]) <$> match' contentp) <|> pure []
   _ <- eolof
-  lowerLines <- many $
+  -- there can be 0 or more nextLines
+  nextLines <- many $
     try (skipSome spacenonewline *> headerp) *> match' contentp <* eolof
-
-  let (textLines, results) = unzip $ sameLine : lowerLines
-      strippedCommentText = T.unlines $ map T.strip textLines
-      result = mconcat results
-  pure (strippedCommentText, result)
+  let
+    -- if there's just a next-line comment, insert an empty same-line comment
+    -- so the next-line comment doesn't get rendered as a same-line comment.
+    sameLine' | null sameLine && not (null nextLines) = [("",mempty)]
+              | otherwise = sameLine 
+    (texts, contents) = unzip $ sameLine' ++ nextLines
+    strippedCommentText = T.unlines $ map T.strip texts
+    commentContent = mconcat contents
+  pure (strippedCommentText, commentContent)
 
   where
     headerp = char ';' *> skipMany spacenonewline
 
 {-# INLINABLE followingcommentp' #-}
 
--- | Parse the text of a (possibly multiline) comment following a journal
--- item.
+-- | Parse the text of a (possibly multiline) comment following a journal item.
+--
+-- >>> rtp followingcommentp ""   -- no comment
+-- Right ""
+-- >>> rtp followingcommentp ";"    -- just a (empty) same-line comment. newline is added
+-- Right "\n"
+-- >>> rtp followingcommentp ";  \n"
+-- Right "\n"
+-- >>> rtp followingcommentp ";\n ;\n"  -- a same-line and a next-line comment
+-- Right "\n\n"
+-- >>> rtp followingcommentp "\n ;\n"  -- just a next-line comment. Insert an empty same-line comment so the next-line comment doesn't become a same-line comment.
+-- Right "\n\n"
+--
 followingcommentp :: TextParser m Text
 followingcommentp =
   fst <$> followingcommentp' (void $ takeWhileP Nothing (/= '\n'))
@@ -1196,25 +1243,25 @@ commenttagsanddatesp mYear = do
 -- default date is provided. A missing year in DATE2 will be inferred
 -- from DATE.
 --
--- >>> either (Left . parseErrorPretty) Right $ rtp (bracketeddatetagsp Nothing) "[2016/1/2=3/4]"
+-- >>> either (Left . customErrorBundlePretty) Right $ rtp (bracketeddatetagsp Nothing) "[2016/1/2=3/4]"
 -- Right [("date",2016-01-02),("date2",2016-03-04)]
 --
--- >>> either (Left . parseErrorPretty) Right $ rtp (bracketeddatetagsp Nothing) "[1]"
+-- >>> either (Left . customErrorBundlePretty) Right $ rtp (bracketeddatetagsp Nothing) "[1]"
 -- Left ...not a bracketed date...
 --
--- >>> either (Left . parseErrorPretty) Right $ rtp (bracketeddatetagsp Nothing) "[2016/1/32]"
--- Left ...1:11:...well-formed but invalid date: 2016/1/32...
+-- >>> either (Left . customErrorBundlePretty) Right $ rtp (bracketeddatetagsp Nothing) "[2016/1/32]"
+-- Left ...1:2:...well-formed but invalid date: 2016/1/32...
 --
--- >>> either (Left . parseErrorPretty) Right $ rtp (bracketeddatetagsp Nothing) "[1/31]"
--- Left ...1:6:...partial date 1/31 found, but the current year is unknown...
+-- >>> either (Left . customErrorBundlePretty) Right $ rtp (bracketeddatetagsp Nothing) "[1/31]"
+-- Left ...1:2:...partial date 1/31 found, but the current year is unknown...
 --
--- >>> either (Left . parseErrorPretty) Right $ rtp (bracketeddatetagsp Nothing) "[0123456789/-.=/-.=]"
+-- >>> either (Left . customErrorBundlePretty) Right $ rtp (bracketeddatetagsp Nothing) "[0123456789/-.=/-.=]"
 -- Left ...1:13:...expecting month or day...
 --
 bracketeddatetagsp
   :: Maybe Year -> TextParser m [(TagName, Day)]
 bracketeddatetagsp mYear1 = do
-  -- pdbg 0 "bracketeddatetagsp"
+  -- dbgparse 0 "bracketeddatetagsp"
   try $ do
     s <- lookAhead
        $ between (char '[') (char ']')
@@ -1245,3 +1292,69 @@ match' :: TextParser m a -> TextParser m (Text, a)
 match' p = do
   (!txt, p) <- match p
   pure (txt, p)
+
+--- * tests
+
+tests_Common = tests "Common" [
+
+  tests "amountp" [
+    test "basic"                  $ expectParseEq amountp "$47.18"     (usd 47.18)
+   ,test "ends with decimal mark" $ expectParseEq amountp "$1."        (usd 1  `withPrecision` 0)
+   ,test "unit price"             $ expectParseEq amountp "$10 @ €0.5" 
+      -- not precise enough:
+      -- (usd 10 `withPrecision` 0 `at` (eur 0.5 `withPrecision` 1)) -- `withStyle` asdecimalpoint=Just '.'
+      amount{
+         acommodity="$"
+        ,aquantity=10 -- need to test internal precision with roundTo ? I think not 
+        ,astyle=amountstyle{asprecision=0, asdecimalpoint=Nothing}
+        ,aprice=UnitPrice $
+          amount{
+             acommodity="€"
+            ,aquantity=0.5
+            ,astyle=amountstyle{asprecision=1, asdecimalpoint=Just '.'}
+            } 
+        } 
+   ,test "total price"            $ expectParseEq amountp "$10 @@ €5"
+      amount{
+         acommodity="$"
+        ,aquantity=10 
+        ,astyle=amountstyle{asprecision=0, asdecimalpoint=Nothing}
+        ,aprice=TotalPrice $
+          amount{
+             acommodity="€"
+            ,aquantity=5
+            ,astyle=amountstyle{asprecision=0, asdecimalpoint=Nothing}
+            } 
+        } 
+    ]
+
+  ,let p = lift (numberp Nothing) :: JournalParser IO (Quantity, Int, Maybe Char, Maybe DigitGroupStyle) in
+   tests "numberp" [
+     test "." $ expectParseEq p "0"          (0, 0, Nothing, Nothing)
+    ,test "." $ expectParseEq p "1"          (1, 0, Nothing, Nothing)
+    ,test "." $ expectParseEq p "1.1"        (1.1, 1, Just '.', Nothing)
+    ,test "." $ expectParseEq p "1,000.1"    (1000.1, 1, Just '.', Just $ DigitGroups ',' [3])
+    ,test "." $ expectParseEq p "1.00.000,1" (100000.1, 1, Just ',', Just $ DigitGroups '.' [3,2])
+    ,test "." $ expectParseEq p "1,000,000"  (1000000, 0, Nothing, Just $ DigitGroups ',' [3,3])  -- could be simplified to [3]
+    ,test "." $ expectParseEq p "1."         (1, 0, Just '.', Nothing)
+    ,test "." $ expectParseEq p "1,"         (1, 0, Just ',', Nothing)
+    ,test "." $ expectParseEq p ".1"         (0.1, 1, Just '.', Nothing)
+    ,test "." $ expectParseEq p ",1"         (0.1, 1, Just ',', Nothing)
+    ,test "." $ expectParseError p "" ""
+    ,test "." $ expectParseError p "1,000.000,1" ""
+    ,test "." $ expectParseError p "1.000,000.1" ""
+    ,test "." $ expectParseError p "1,000.000.1" ""
+    ,test "." $ expectParseError p "1,,1" ""
+    ,test "." $ expectParseError p "1..1" ""
+    ,test "." $ expectParseError p ".1," ""
+    ,test "." $ expectParseError p ",1." ""
+    ]
+  
+  ,tests "spaceandamountormissingp" [
+     test "space and amount" $ expectParseEq spaceandamountormissingp " $47.18" (Mixed [usd 47.18])
+    ,test "empty string" $ expectParseEq spaceandamountormissingp "" missingmixedamt
+    ,_test "just space" $ expectParseEq spaceandamountormissingp " " missingmixedamt  -- XXX should it ?
+    -- ,test "just amount" $ expectParseError spaceandamountormissingp "$47.18" ""  -- succeeds, consuming nothing
+    ]
+
+  ]
