@@ -30,8 +30,6 @@ module Hledger.Read.TimedotReader (
   reader,
   -- * Misc other exports
   timedotfilep,
-  -- * Tests
-  tests_Hledger_Read_TimedotReader
 )
 where
 import Prelude ()
@@ -43,19 +41,18 @@ import Data.Char (isSpace)
 import Data.List (foldl')
 import Data.Maybe
 import Data.Text (Text)
-import Test.HUnit
 import Text.Megaparsec hiding (parse)
 import Text.Megaparsec.Char
 
 import Hledger.Data
 import Hledger.Read.Common
-import Hledger.Utils hiding (ptrace)
+import Hledger.Utils hiding (traceParse)
 
 -- easier to toggle this here sometimes
--- import qualified Hledger.Utils (ptrace)
--- ptrace = Hledger.Utils.ptrace
-ptrace :: Monad m => a -> m a
-ptrace = return
+-- import qualified Hledger.Utils (parsertrace)
+-- parsertrace = Hledger.Utils.parsertrace
+traceParse :: Monad m => a -> m a
+traceParse = return
 
 reader :: Reader
 reader = Reader
@@ -67,7 +64,7 @@ reader = Reader
 
 -- | Parse and post-process a "Journal" from the timedot format, or give an error.
 parse :: InputOpts -> FilePath -> Text -> ExceptT String IO Journal
-parse = parseAndFinaliseJournal timedotfilep
+parse = parseAndFinaliseJournal' timedotfilep
 
 timedotfilep :: JournalParser m ParsedJournal
 timedotfilep = do many timedotfileitemp
@@ -76,7 +73,7 @@ timedotfilep = do many timedotfileitemp
     where
       timedotfileitemp :: JournalParser m ()
       timedotfileitemp = do
-        ptrace "timedotfileitemp"
+        traceParse "timedotfileitemp"
         choice [
           void $ lift emptyorcommentlinep
          ,timedotdayp >>= \ts -> modify' (addTransactions ts)
@@ -94,7 +91,7 @@ addTransactions ts j = foldl' (flip ($)) j (map addTransaction ts)
 -- @
 timedotdayp :: JournalParser m [Transaction]
 timedotdayp = do
-  ptrace " timedotdayp"
+  traceParse " timedotdayp"
   d <- datep <* lift eolof
   es <- catMaybes <$> many (const Nothing <$> try (lift emptyorcommentlinep) <|>
                             Just <$> (notFollowedBy datep >> timedotentryp))
@@ -106,8 +103,8 @@ timedotdayp = do
 -- @
 timedotentryp :: JournalParser m Transaction
 timedotentryp = do
-  ptrace "  timedotentryp"
-  pos <- genericSourcePos <$> getPosition
+  traceParse "  timedotentryp"
+  pos <- genericSourcePos <$> getSourcePos
   lift (skipMany spacenonewline)
   a <- modifiedaccountnamep
   lift (skipMany spacenonewline)
@@ -173,7 +170,3 @@ timedotdotsp :: JournalParser m Quantity
 timedotdotsp = do
   dots <- filter (not.isSpace) <$> many (oneOf (". " :: [Char]))
   return $ (/4) $ fromIntegral $ length dots
-
-tests_Hledger_Read_TimedotReader = TestList [
- ]
-
