@@ -16,7 +16,7 @@ module Hledger.Reports.MultiBalanceReports (
   tableAsText,
 
   -- -- * Tests
-  tests_Hledger_Reports_MultiBalanceReport
+  tests_MultiBalanceReports
 )
 where
 
@@ -25,13 +25,12 @@ import Data.Maybe
 import Data.Ord
 import Data.Time.Calendar
 import Safe
-import Test.HUnit
 import Text.Tabular as T
 import Text.Tabular.AsciiWide
 
 import Hledger.Data
 import Hledger.Query
-import Hledger.Utils
+import Hledger.Utils 
 import Hledger.Read (mamountp')
 import Hledger.Reports.ReportOptions
 import Hledger.Reports.BalanceReport
@@ -69,11 +68,11 @@ type MultiBalanceReportRow    = (AccountName, AccountName, Int, [MixedAmount], M
 type MultiBalanceReportTotals = ([MixedAmount], MixedAmount, MixedAmount) -- (Totals list, sum of totals, average of totals)
 
 instance Show MultiBalanceReport where
-    -- use ppShow to break long lists onto multiple lines
-    -- we add some bogus extra shows here to help ppShow parse the output
+    -- use pshow (pretty-show's ppShow) to break long lists onto multiple lines
+    -- we add some bogus extra shows here to help it parse the output
     -- and wrap tuples and lists properly
     show (MultiBalanceReport (spans, items, totals)) =
-        "MultiBalanceReport (ignore extra quotes):\n" ++ ppShow (show spans, map show items, totals)
+        "MultiBalanceReport (ignore extra quotes):\n" ++ pshow (show spans, map show items, totals)
 
 -- type alias just to remind us which AccountNames might be depth-clipped, below.
 type ClippedAccountName = AccountName
@@ -94,15 +93,37 @@ multiBalanceReport opts q j =
       depthless  = dbg1 "depthless" . filterQuery (not . queryIsDepth)
       datelessq  = dbg1 "datelessq"  $ filterQuery (not . queryIsDateOrDate2) q
       dateqcons  = if date2_ opts then Date2 else Date
-      precedingq = dbg1 "precedingq" $ And [datelessq, dateqcons $ DateSpan Nothing (spanStart reportspan)]
-      requestedspan  = dbg1 "requestedspan"  $ queryDateSpan (date2_ opts) q                              -- span specified by -b/-e/-p options and query args
-      requestedspan' = dbg1 "requestedspan'" $ requestedspan `spanDefaultsFrom` journalDateSpan (date2_ opts) j  -- if open-ended, close it using the journal's end dates
-      intervalspans  = dbg1 "intervalspans"  $ splitSpan (interval_ opts) requestedspan'           -- interval spans enclosing it
-      reportspan     = dbg1 "reportspan"     $ DateSpan (maybe Nothing spanStart $ headMay intervalspans) -- the requested span enlarged to a whole number of intervals
-                                                       (maybe Nothing spanEnd   $ lastMay intervalspans)
-      newdatesq = dbg1 "newdateq" $ dateqcons reportspan
-      reportq  = dbg1 "reportq" $ depthless $ And [datelessq, newdatesq] -- user's query enlarged to whole intervals and with no depth limit
-
+      -- The date span specified by -b/-e/-p options and query args if any.
+      requestedspan  = dbg1 "requestedspan"  $ queryDateSpan (date2_ opts) q
+      -- If the requested span is open-ended, close it using the journal's end dates.
+      -- This can still be the null (open) span if the journal is empty.
+      requestedspan' = dbg1 "requestedspan'" $ requestedspan `spanDefaultsFrom` journalDateSpan (date2_ opts) j
+      -- The list of interval spans enclosing the requested span.
+      -- This list can be empty if the journal was empty,
+      -- or if hledger-ui has added its special date:-tomorrow to the query
+      -- and all txns are in the future.
+      intervalspans  = dbg1 "intervalspans"  $ splitSpan (interval_ opts) requestedspan'           
+      -- The requested span enlarged to enclose a whole number of intervals.
+      -- This can be the null span if there were no intervals. 
+      reportspan     = dbg1 "reportspan"     $ DateSpan (maybe Nothing spanStart $ headMay intervalspans)
+                                                        (maybe Nothing spanEnd   $ lastMay intervalspans)
+      -- The user's query with no depth limit, and expanded to the report span
+      -- if there is one (otherwise any date queries are left as-is, which
+      -- handles the hledger-ui+future txns case above).
+      reportq   = dbg1 "reportq" $ depthless $ 
+        if reportspan == nulldatespan 
+        then q 
+        else And [datelessq, reportspandatesq]
+          where
+            reportspandatesq = dbg1 "reportspandatesq" $ dateqcons reportspan
+      -- q projected back before the report start date, to calculate starting balances.
+      -- When there's no report start date, in case there are future txns (the hledger-ui case above),
+      -- we use emptydatespan to make sure they aren't counted as starting balance.  
+      startbalq = dbg1 "startbalq" $ And [datelessq, dateqcons precedingspan]
+        where
+          precedingspan = case spanStart reportspan of
+                            Just d  -> DateSpan Nothing (Just d)
+                            Nothing -> emptydatespan 
       ps :: [Posting] =
           dbg1 "ps" $
           journalPostings $
@@ -140,7 +161,7 @@ multiBalanceReport opts q j =
       -- starting balances and accounts from transactions before the report start date
       startacctbals = dbg1 "startacctbals" $ map (\(a,_,_,b) -> (a,b)) startbalanceitems
           where
-            (startbalanceitems,_) = dbg1 "starting balance report" $ balanceReport opts' precedingq j
+            (startbalanceitems,_) = dbg1 "starting balance report" $ balanceReport opts' startbalq j
                                     where
                                       opts' | tree_ opts = opts{no_elide_=True}
                                             | otherwise  = opts{accountlistmode_=ALFlat}
@@ -176,64 +197,45 @@ multiBalanceReport opts q j =
            , empty_ opts || depth == 0 || any (not . isZeroMixedAmount) displayedBals
            ]
 
+      -- TODO TBD: is it always ok to sort report rows after report has been generated ?
+      -- Or does sorting sometimes need to be done as part of the report generation ?  
       sorteditems :: [MultiBalanceReportRow] =
         dbg1 "sorteditems" $
         sortitems items
         where
           sortitems
-            | sort_amount_ opts && accountlistmode_ opts == ALTree       = sortTreeMultiBalanceReportRowsByAmount
-            | sort_amount_ opts                                          = sortFlatMultiBalanceReportRowsByAmount
-            | not (sort_amount_ opts) && accountlistmode_ opts == ALTree = sortTreeMultiBalanceReportRowsByAccountCodeAndName
-            | otherwise                                                  = sortFlatMultiBalanceReportRowsByAccountCodeAndName
+            | sort_amount_ opts && accountlistmode_ opts == ALTree       = sortTreeMBRByAmount
+            | sort_amount_ opts                                          = sortFlatMBRByAmount
+            | otherwise                                                  = sortMBRByAccountDeclaration
             where
-              -- Sort the report rows, representing a flat account list, by row total. 
-              sortFlatMultiBalanceReportRowsByAmount = sortBy (maybeflip $ comparing fifth6)
-                where
-                  maybeflip = if normalbalance_ opts == Just NormallyNegative then id else flip
-
               -- Sort the report rows, representing a tree of accounts, by row total at each level.
-              -- To do this we recreate an Account tree with the row totals as balances, 
-              -- so we can do a hierarchical sort, flatten again, and then reorder the  
-              -- report rows similarly. Yes this is pretty long winded. 
-              sortTreeMultiBalanceReportRowsByAmount rows = sortedrows
+              -- Similar to sortMBRByAccountDeclaration/sortAccountNamesByDeclaration.
+              sortTreeMBRByAmount rows = sortedrows
                 where
                   anamesandrows = [(first6 r, r) | r <- rows]
                   anames = map fst anamesandrows
                   atotals = [(a,tot) | (a,_,_,_,tot,_) <- rows]
-                  nametree = treeFromPaths $ map expandAccountName anames
-                  accounttree = nameTreeToAccount "root" nametree
+                  accounttree = accountTree "root" anames
                   accounttreewithbals = mapAccounts setibalance accounttree
                     where
-                      -- this error should not happen, but it's ugly TODO 
-                      setibalance a = a{aibalance=fromMaybe (error "sortTreeMultiBalanceReportRowsByAmount 1") $ lookup (aname a) atotals}
+                      -- should not happen, but it's dangerous; TODO 
+                      setibalance a = a{aibalance=fromMaybe (error "sortTreeMBRByAmount 1") $ lookup (aname a) atotals}
                   sortedaccounttree = sortAccountTreeByAmount (fromMaybe NormallyPositive $ normalbalance_ opts) accounttreewithbals
-                  sortedaccounts = drop 1 $ flattenAccounts sortedaccounttree
-                  -- dropped the root account, also ignore any parent accounts not in rows
-                  sortedrows = concatMap (\a -> maybe [] (:[]) $ lookup (aname a) anamesandrows) sortedaccounts 
+                  sortedanames = map aname $ drop 1 $ flattenAccounts sortedaccounttree
+                  sortedrows = sortAccountItemsLike sortedanames anamesandrows 
 
-              -- Sort the report rows by account code if any, with the empty account code coming last, then account name. 
-              -- TODO keep children below their parent. Have to convert to tree ? 
-              sortFlatMultiBalanceReportRowsByAccountCodeAndName = sortBy (comparing acodeandname)
+              -- Sort the report rows, representing a flat account list, by row total. 
+              sortFlatMBRByAmount = sortBy (maybeflip $ comparing (normaliseMixedAmountSquashPricesForDisplay . fifth6))
                 where
-                  acodeandname r = (acode', aname)
-                    where
-                      aname = first6 r
-                      macode = fromMaybe Nothing $ lookup aname $ jaccounts j
-                      acode' = fromMaybe maxBound macode 
+                  maybeflip = if normalbalance_ opts == Just NormallyNegative then id else flip
 
-              -- Sort the report rows, representing a tree of accounts, by account code and then account name at each level.
-              -- Convert a tree of account names, look up the account codes, sort and flatten the tree, reorder the rows.
-              sortTreeMultiBalanceReportRowsByAccountCodeAndName rows = sortedrows
-                where
+              -- Sort the report rows by account declaration order then account name. 
+              sortMBRByAccountDeclaration rows = sortedrows
+                where 
                   anamesandrows = [(first6 r, r) | r <- rows]
                   anames = map fst anamesandrows
-                  nametree = treeFromPaths $ map expandAccountName anames
-                  accounttree = nameTreeToAccount "root" nametree
-                  accounttreewithcodes = mapAccounts (accountSetCodeFrom j) accounttree
-                  sortedaccounttree = sortAccountTreeByAccountCodeAndName accounttreewithcodes
-                  sortedaccounts = drop 1 $ flattenAccounts sortedaccounttree
-                  -- dropped the root account, also ignore any parent accounts not in rows
-                  sortedrows = concatMap (\a -> maybe [] (:[]) $ lookup (aname a) anamesandrows) sortedaccounts 
+                  sortedanames = sortAccountNamesByDeclaration j (tree_ opts) anames
+                  sortedrows = sortAccountItemsLike sortedanames anamesandrows 
 
       totals :: [MixedAmount] =
           -- dbg1 "totals" $
@@ -285,53 +287,6 @@ balanceReportFromMultiBalanceReport opts q j = (rows', total)
     total = headDef nullmixedamt totals
 
 
-tests_multiBalanceReport =
-  let
-    (opts,journal) `gives` r = do
-      let (eitems, etotal) = r
-          (MultiBalanceReport (_, aitems, atotal)) = multiBalanceReport opts (queryFromOpts nulldate opts) journal
-          showw (acct,acct',indent,lAmt,amt,amt') = (acct, acct', indent, map showMixedAmountDebug lAmt, showMixedAmountDebug amt, showMixedAmountDebug amt')
-      assertEqual "items" (map showw eitems) (map showw aitems)
-      assertEqual "total" (showMixedAmountDebug etotal) ((\(_, b, _) -> showMixedAmountDebug b) atotal) -- we only check the sum of the totals
-    usd0 = usd 0
-    amount0 = Amount {acommodity="$", aquantity=0, aprice=NoPrice, astyle=AmountStyle {ascommodityside = L, ascommodityspaced = False, asprecision = 2, asdecimalpoint = Just '.', asdigitgroups = Nothing}, amultiplier=False}
-  in [
-   "multiBalanceReport with no args on null journal" ~: do
-   (defreportopts, nulljournal) `gives` ([], Mixed [nullamt])
-
-   ,"multiBalanceReport with -H on a populated period" ~: do
-    (defreportopts{period_= PeriodBetween (fromGregorian 2008 1 1) (fromGregorian 2008 1 2), balancetype_=HistoricalBalance}, samplejournal) `gives`
-     (
-      [
-       ("assets:bank:checking","checking",3, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=1}])
-      ,("income:salary","salary",2, [mamountp' "$-1.00"], mamountp' "$-1.00",Mixed [amount0 {aquantity=(-1)}])
-      ],
-      Mixed [usd0])
-
-   ,"multiBalanceReport tests the ability to have a valid history on an empty period" ~: do
-    (defreportopts{period_= PeriodBetween (fromGregorian 2008 1 2) (fromGregorian 2008 1 3), balancetype_=HistoricalBalance}, samplejournal) `gives`
-     (
-      [
-       ("assets:bank:checking","checking",3, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=1}])
-      ,("income:salary","salary",2, [mamountp' "$-1.00"], mamountp' "$-1.00",Mixed [amount0 {aquantity=(-1)}])
-      ],
-      Mixed [usd0])
-
-   ,"multiBalanceReport tests the ability to have a valid history on an empty period (More complex)" ~: do
-    (defreportopts{period_= PeriodBetween (fromGregorian 2009 1 1) (fromGregorian 2009 1 2), balancetype_=HistoricalBalance}, samplejournal) `gives`
-     (
-      [
-      ("assets:bank:checking","checking",3, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=1}])
-      ,("assets:bank:saving","saving",3, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=1}])
-      ,("assets:cash","cash",2, [mamountp' "$-2.00"], mamountp' "$-2.00",Mixed [amount0 {aquantity=(-2)}])
-      ,("expenses:food","food",2, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=(1)}])
-      ,("expenses:supplies","supplies",2, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=(1)}])
-      ,("income:gifts","gifts",2, [mamountp' "$-1.00"], mamountp' "$-1.00",Mixed [amount0 {aquantity=(-1)}])
-      ,("income:salary","salary",2, [mamountp' "$-1.00"], mamountp' "$-1.00",Mixed [amount0 {aquantity=(-1)}])
-      ],
-      Mixed [usd0])
-  ]
-
 -- common rendering helper, XXX here for now
 
 tableAsText :: ReportOpts -> (a -> String) -> Table String String a -> String
@@ -348,6 +303,53 @@ tableAsText (ReportOpts{pretty_tables_ = pretty}) showcell =
         acctswidth = maximum' $ map strWidth (headerContents l)
         l'         = padRightWide acctswidth <$> l
 
-tests_Hledger_Reports_MultiBalanceReport :: Test
-tests_Hledger_Reports_MultiBalanceReport = TestList
-  tests_multiBalanceReport
+-- tests
+
+tests_MultiBalanceReports = tests "MultiBalanceReports" [
+  let
+    (opts,journal) `gives` r = do
+      let (eitems, etotal) = r
+          (MultiBalanceReport (_, aitems, atotal)) = multiBalanceReport opts (queryFromOpts nulldate opts) journal
+          showw (acct,acct',indent,lAmt,amt,amt') = (acct, acct', indent, map showMixedAmountDebug lAmt, showMixedAmountDebug amt, showMixedAmountDebug amt')
+      (map showw aitems) `is` (map showw eitems)
+      ((\(_, b, _) -> showMixedAmountDebug b) atotal) `is` (showMixedAmountDebug etotal) -- we only check the sum of the totals
+    usd0 = usd 0
+    amount0 = Amount {acommodity="$", aquantity=0, aprice=NoPrice, astyle=AmountStyle {ascommodityside = L, ascommodityspaced = False, asprecision = 2, asdecimalpoint = Just '.', asdigitgroups = Nothing}, amultiplier=False}
+  in 
+   tests "multiBalanceReport" [
+      test "null journal"  $
+      (defreportopts, nulljournal) `gives` ([], Mixed [nullamt])
+  
+     ,test "with -H on a populated period"  $
+      (defreportopts{period_= PeriodBetween (fromGregorian 2008 1 1) (fromGregorian 2008 1 2), balancetype_=HistoricalBalance}, samplejournal) `gives`
+       (
+        [
+         ("assets:bank:checking","checking",3, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=1}])
+        ,("income:salary","salary",2, [mamountp' "$-1.00"], mamountp' "$-1.00",Mixed [amount0 {aquantity=(-1)}])
+        ],
+        Mixed [usd0])
+  
+     ,test "a valid history on an empty period"  $
+      (defreportopts{period_= PeriodBetween (fromGregorian 2008 1 2) (fromGregorian 2008 1 3), balancetype_=HistoricalBalance}, samplejournal) `gives`
+       (
+        [
+         ("assets:bank:checking","checking",3, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=1}])
+        ,("income:salary","salary",2, [mamountp' "$-1.00"], mamountp' "$-1.00",Mixed [amount0 {aquantity=(-1)}])
+        ],
+        Mixed [usd0])
+  
+     ,test "a valid history on an empty period (more complex)"  $
+      (defreportopts{period_= PeriodBetween (fromGregorian 2009 1 1) (fromGregorian 2009 1 2), balancetype_=HistoricalBalance}, samplejournal) `gives`
+       (
+        [
+        ("assets:bank:checking","checking",3, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=1}])
+        ,("assets:bank:saving","saving",3, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=1}])
+        ,("assets:cash","cash",2, [mamountp' "$-2.00"], mamountp' "$-2.00",Mixed [amount0 {aquantity=(-2)}])
+        ,("expenses:food","food",2, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=(1)}])
+        ,("expenses:supplies","supplies",2, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=(1)}])
+        ,("income:gifts","gifts",2, [mamountp' "$-1.00"], mamountp' "$-1.00",Mixed [amount0 {aquantity=(-1)}])
+        ,("income:salary","salary",2, [mamountp' "$-1.00"], mamountp' "$-1.00",Mixed [amount0 {aquantity=(-1)}])
+        ],
+        Mixed [usd0])
+    ]
+ ]
