@@ -4,6 +4,7 @@ Standard imports and utilities which are useful everywhere, or needed low
 in the module hierarchy. This is the bottom of hledger's module graph.
 
 -}
+{-# LANGUAGE OverloadedStrings, LambdaCase #-}
 
 module Hledger.Utils (---- provide these frequently used modules - or not, for clearer api:
                           -- module Control.Monad,
@@ -14,7 +15,6 @@ module Hledger.Utils (---- provide these frequently used modules - or not, for c
                           -- module Data.Time.LocalTime,
                           -- module Data.Tree,
                           -- module Text.RegexPR,
-                          -- module Test.HUnit,
                           -- module Text.Printf,
                           ---- all of this one:
                           module Hledger.Utils,
@@ -33,7 +33,6 @@ module Hledger.Utils (---- provide these frequently used modules - or not, for c
                           -- the rest need to be done in each module I think
                           )
 where
-import Test.HUnit
 
 import Control.Monad (liftM, when)
 -- import Data.Char
@@ -129,9 +128,12 @@ isLeft _        = False
 isRight :: Either a b -> Bool
 isRight = not . isLeft
 
--- | Apply a function the specified number of times. Possibly uses O(n) stack ?
+-- | Apply a function the specified number of times,
+-- which should be > 0 (otherwise does nothing).
+-- Possibly uses O(n) stack ?
 applyN :: Int -> (a -> a) -> a -> a
-applyN n f = (!! n) . iterate f
+applyN n f | n < 1     = id
+           | otherwise = (!! n) . iterate f
 -- from protolude, compare
 -- applyN :: Int -> (a -> a) -> a -> a
 -- applyN n f = X.foldr (.) identity (X.replicate n f)
@@ -141,12 +143,15 @@ applyN n f = (!! n) . iterate f
 -- Can raise an error.
 expandPath :: FilePath -> FilePath -> IO FilePath -- general type sig for use in reader parsers
 expandPath _ "-" = return "-"
-expandPath curdir p = (if isRelative p then (curdir </>) else id) `liftM` expandPath' p
-  where
-    expandPath' ('~':'/':p)  = (</> p) <$> getHomeDirectory
-    expandPath' ('~':'\\':p) = (</> p) <$> getHomeDirectory
-    expandPath' ('~':_)      = ioError $ userError "~USERNAME in paths is not supported"
-    expandPath' p            = return p
+expandPath curdir p = (if isRelative p then (curdir </>) else id) `liftM` expandHomePath p
+
+-- | Expand user home path indicated by tilde prefix
+expandHomePath :: FilePath -> IO FilePath
+expandHomePath = \case
+    ('~':'/':p)  -> (</> p) <$> getHomeDirectory
+    ('~':'\\':p) -> (</> p) <$> getHomeDirectory
+    ('~':_)      -> ioError $ userError "~USERNAME in paths is not supported"
+    p            -> return p
 
 firstJust ms = case dropWhile (==Nothing) ms of
     [] -> Nothing
@@ -215,7 +220,6 @@ sequence' ms = do
 mapM' :: Monad f => (a -> f b) -> [a] -> f [b]
 mapM' f = sequence' . map f
 
-tests_Hledger_Utils :: Test
-tests_Hledger_Utils = TestList [
-    tests_Hledger_Utils_Text
-    ]
\ No newline at end of file
+tests_Utils = tests "Utils" [
+  tests_Text
+  ]
