@@ -54,10 +54,10 @@ module Hledger.Utils.Text
  -- fitStringMulti,
   textPadLeftWide,
   textPadRightWide,
-  tests_Hledger_Utils_Text
+  -- -- * tests
+  tests_Text
   )
 where
-import Test.HUnit
 
 -- import Data.Char
 import Data.List
@@ -72,6 +72,7 @@ import qualified Data.Text as T
 -- import Hledger.Utils.Parse
 -- import Hledger.Utils.Regex
 import Hledger.Utils.String (charWidth)
+import Hledger.Utils.Test
 
 -- lowercase, uppercase :: String -> String
 -- lowercase = map toLower
@@ -419,12 +420,14 @@ textWidth s = maximum $ map (T.foldr (\a b -> charWidth a + b) 0) $ T.lines s
 --         | otherwise                        -> 1
 
 
-tests_Hledger_Utils_Text = TestList [
-      quoteIfSpaced "a'a" ~?= "a'a"
-    , quoteIfSpaced "a\"a" ~?= "a\"a"
-    , quoteIfSpaced "a a" ~?= "\"a a\""
-    , quoteIfSpaced "mimi's cafe" ~?= "\"mimi's cafe\""
-    , quoteIfSpaced "\"alex\" cafe" ~?= "\"\\\"alex\\\" cafe\""
-    , quoteIfSpaced "le'shan's cafe" ~?= "\"le'shan's cafe\""
-    , quoteIfSpaced "\"be'any's\" cafe" ~?= "\"\\\"be'any's\\\" cafe\""
-    ]
\ No newline at end of file
+tests_Text = tests "Text" [
+   tests "quoteIfSpaced" [
+     quoteIfSpaced "a'a" `is` "a'a"
+    ,quoteIfSpaced "a\"a" `is` "a\"a"              
+    ,quoteIfSpaced "a a" `is` "\"a a\""               
+    ,quoteIfSpaced "mimi's cafe" `is` "\"mimi's cafe\""       
+    ,quoteIfSpaced "\"alex\" cafe" `is` "\"\\\"alex\\\" cafe\""     
+    ,quoteIfSpaced "le'shan's cafe" `is` "\"le'shan's cafe\""    
+    ,quoteIfSpaced "\"be'any's\" cafe" `is` "\"\\\"be'any's\\\" cafe\"" 
+    ] 
+  ]
